@@ -7,41 +7,37 @@ import (
 )
 
 func TestErrGroup(t *testing.T) {
-	testCases := []struct {
-		errs []error
-		exp  string
-	}{
-		{nil, ""},
-		{[]error{errors.New("timeout")}, "errors(1): timeout"},
-		{
-			[]error{errors.New("timeout"), errors.New("deadline")},
-			"errors(2): timeout\ndeadline",
-		},
-	}
-	for _, tc := range testCases {
-		eg := new(ErrGroup)
-		for _, err := range tc.errs {
+	f := func(errs []error, resultExpected string) {
+		t.Helper()
+
+		eg := &ErrGroup{}
+		for _, err := range errs {
 			eg.Add(err)
 		}
-		if len(tc.errs) == 0 {
+		if len(errs) == 0 {
 			if eg.Err() != nil {
 				t.Fatalf("expected to get nil error")
 			}
-			continue
+			return
 		}
 		if eg.Err() == nil {
 			t.Fatalf("expected to get non-nil error")
 		}
-		if eg.Error() != tc.exp {
-			t.Fatalf("expected to have: \n%q\ngot:\n%q", tc.exp, eg.Error())
+		result := eg.Error()
+		if result != resultExpected {
+			t.Fatalf("unexpected result\ngot\n%v\nwant\n%v", result, resultExpected)
 		}
 	}
+
+	f(nil, "")
+	f([]error{errors.New("timeout")}, "errors(1): timeout")
+	f([]error{errors.New("timeout"), errors.New("deadline")}, "errors(2): timeout\ndeadline")
 }
 
 // TestErrGroupConcurrent supposed to test concurrent
 // use of error group.
 // Should be executed with -race flag
-func TestErrGroupConcurrent(t *testing.T) {
+func TestErrGroupConcurrent(_ *testing.T) {
 	eg := new(ErrGroup)
 
 	const writersN = 4
