@@ -10,12 +10,14 @@ import (
 	"sync"
 	"time"
 
-	"github.com/VictoriaMetrics/VictoriaMetrics/app/vmselect/bufferedwriter"
 	"github.com/VictoriaMetrics/VictoriaMetrics/app/vmselect/netstorage"
 	"github.com/VictoriaMetrics/VictoriaMetrics/app/vmselect/searchutils"
+	"github.com/VictoriaMetrics/VictoriaMetrics/lib/bufferedwriter"
+	"github.com/VictoriaMetrics/VictoriaMetrics/lib/httputils"
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/logger"
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/storage"
 	"github.com/VictoriaMetrics/metrics"
+	"github.com/VictoriaMetrics/metricsql"
 )
 
 var maxTagValueSuffixes = flag.Int("search.maxTagValueSuffixesPerSearch", 100e3, "The maximum number of tag value suffixes returned from /metrics/find")
@@ -45,7 +47,7 @@ func MetricsFindHandler(startTime time.Time, w http.ResponseWriter, r *http.Requ
 	if len(delimiter) > 1 {
 		return fmt.Errorf("`delimiter` query arg must contain only a single char")
 	}
-	if searchutils.GetBool(r, "automatic_variants") {
+	if httputils.GetBool(r, "automatic_variants") {
 		// See https://github.com/graphite-project/graphite-web/blob/bb9feb0e6815faa73f538af6ed35adea0fb273fd/webapp/graphite/metrics/views.py#L152
 		query = addAutomaticVariants(query, delimiter)
 	}
@@ -56,19 +58,19 @@ func MetricsFindHandler(startTime time.Time, w http.ResponseWriter, r *http.Requ
 			query += "*"
 		}
 	}
-	leavesOnly := searchutils.GetBool(r, "leavesOnly")
-	wildcards := searchutils.GetBool(r, "wildcards")
+	leavesOnly := httputils.GetBool(r, "leavesOnly")
+	wildcards := httputils.GetBool(r, "wildcards")
 	label := r.FormValue("label")
 	if label == "__name__" {
 		label = ""
 	}
 	jsonp := r.FormValue("jsonp")
-	from, err := searchutils.GetTime(r, "from", 0)
+	from, err := httputils.GetTime(r, "from", 0)
 	if err != nil {
 		return err
 	}
 	ct := startTime.UnixNano() / 1e6
-	until, err := searchutils.GetTime(r, "until", ct)
+	until, err := httputils.GetTime(r, "until", ct)
 	if err != nil {
 		return err
 	}
@@ -83,7 +85,7 @@ func MetricsFindHandler(startTime time.Time, w http.ResponseWriter, r *http.Requ
 	if leavesOnly {
 		paths = filterLeaves(paths, delimiter)
 	}
-	paths = deduplicatePaths(paths, delimiter)
+	paths = deduplicatePaths(paths)
 	sortPaths(paths, delimiter)
 	contentType := getContentType(jsonp)
 	w.Header().Set("Content-Type", contentType)
@@ -97,7 +99,7 @@ func MetricsFindHandler(startTime time.Time, w http.ResponseWriter, r *http.Requ
 	return nil
 }
 
-func deduplicatePaths(paths []string, delimiter string) []string {
+func deduplicatePaths(paths []string) []string {
 	if len(paths) == 0 {
 		return nil
 	}
@@ -123,8 +125,8 @@ func MetricsExpandHandler(startTime time.Time, w http.ResponseWriter, r *http.Re
 	if len(queries) == 0 {
 		return fmt.Errorf("missing `query` arg")
 	}
-	groupByExpr := searchutils.GetBool(r, "groupByExpr")
-	leavesOnly := searchutils.GetBool(r, "leavesOnly")
+	groupByExpr := httputils.GetBool(r, "groupByExpr")
+	leavesOnly := httputils.GetBool(r, "leavesOnly")
 	label := r.FormValue("label")
 	if label == "__name__" {
 		label = ""
@@ -137,12 +139,12 @@ func MetricsExpandHandler(startTime time.Time, w http.ResponseWriter, r *http.Re
 		return fmt.Errorf("`delimiter` query arg must contain only a single char")
 	}
 	jsonp := r.FormValue("jsonp")
-	from, err := searchutils.GetTime(r, "from", 0)
+	from, err := httputils.GetTime(r, "from", 0)
 	if err != nil {
 		return err
 	}
 	ct := startTime.UnixNano() / 1e6
-	until, err := searchutils.GetTime(r, "until", ct)
+	until, err := httputils.GetTime(r, "until", ct)
 	if err != nil {
 		return err
 	}
@@ -349,7 +351,7 @@ func getRegexpForQuery(query string, delimiter byte) (*regexp.Regexp, error) {
 	if len(tail) > 0 {
 		return nil, fmt.Errorf("unexpected tail left after parsing query %q; tail: %q", query, tail)
 	}
-	re, err := regexp.Compile(rs)
+	re, err := metricsql.CompileRegexp(rs)
 	regexpCache[k] = &regexpCacheEntry{
 		re:  re,
 		err: err,
