@@ -1,6 +1,7 @@
 package promql
 
 import (
+	"fmt"
 	"testing"
 
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/fs"
@@ -9,13 +10,13 @@ import (
 )
 
 func TestRollupResultCacheInitStop(t *testing.T) {
-	t.Run("inmemory", func(t *testing.T) {
+	t.Run("inmemory", func(_ *testing.T) {
 		for i := 0; i < 5; i++ {
 			InitRollupResultCache("")
 			StopRollupResultCache()
 		}
 	})
-	t.Run("file-based", func(t *testing.T) {
+	t.Run("file-based", func(_ *testing.T) {
 		cacheFilePath := "test-rollup-result-cache"
 		for i := 0; i < 3; i++ {
 			InitRollupResultCache(cacheFilePath)
@@ -33,17 +34,22 @@ func TestRollupResultCache(t *testing.T) {
 	ResetRollupResultCache()
 	window := int64(456)
 	ec := &EvalConfig{
-		Start: 1000,
-		End:   2000,
-		Step:  200,
+		Start:              1000,
+		End:                2000,
+		Step:               200,
+		MaxPointsPerSeries: 1e4,
 
 		MayCache: true,
 	}
 	me := &metricsql.MetricExpr{
-		LabelFilters: []metricsql.LabelFilter{{
-			Label: "aaa",
-			Value: "xxx",
-		}},
+		LabelFilterss: [][]metricsql.LabelFilter{
+			{
+				{
+					Label: "aaa",
+					Value: "xxx",
+				},
+			},
+		},
 	}
 	fe := &metricsql.FuncExpr{
 		Name: "foo",
@@ -56,7 +62,7 @@ func TestRollupResultCache(t *testing.T) {
 
 	// Try obtaining an empty value.
 	t.Run("empty", func(t *testing.T) {
-		tss, newStart := rollupResultCacheV.Get(nil, ec, fe, window)
+		tss, newStart := rollupResultCacheV.GetSeries(nil, ec, fe, window)
 		if newStart != ec.Start {
 			t.Fatalf("unexpected newStart; got %d; want %d", newStart, ec.Start)
 		}
@@ -74,8 +80,8 @@ func TestRollupResultCache(t *testing.T) {
 				Values:     []float64{0, 1, 2},
 			},
 		}
-		rollupResultCacheV.Put(nil, ec, fe, window, tss)
-		tss, newStart := rollupResultCacheV.Get(nil, ec, fe, window)
+		rollupResultCacheV.PutSeries(nil, ec, fe, window, tss)
+		tss, newStart := rollupResultCacheV.GetSeries(nil, ec, fe, window)
 		if newStart != 1400 {
 			t.Fatalf("unexpected newStart; got %d; want %d", newStart, 1400)
 		}
@@ -95,8 +101,8 @@ func TestRollupResultCache(t *testing.T) {
 				Values:     []float64{0, 1, 2},
 			},
 		}
-		rollupResultCacheV.Put(nil, ec, ae, window, tss)
-		tss, newStart := rollupResultCacheV.Get(nil, ec, ae, window)
+		rollupResultCacheV.PutSeries(nil, ec, ae, window, tss)
+		tss, newStart := rollupResultCacheV.GetSeries(nil, ec, ae, window)
 		if newStart != 1400 {
 			t.Fatalf("unexpected newStart; got %d; want %d", newStart, 1400)
 		}
@@ -118,8 +124,8 @@ func TestRollupResultCache(t *testing.T) {
 				Values:     []float64{333, 0, 1, 2},
 			},
 		}
-		rollupResultCacheV.Put(nil, ec, fe, window, tss)
-		tss, newStart := rollupResultCacheV.Get(nil, ec, fe, window)
+		rollupResultCacheV.PutSeries(nil, ec, fe, window, tss)
+		tss, newStart := rollupResultCacheV.GetSeries(nil, ec, fe, window)
 		if newStart != 1000 {
 			t.Fatalf("unexpected newStart; got %d; want %d", newStart, 1000)
 		}
@@ -137,8 +143,8 @@ func TestRollupResultCache(t *testing.T) {
 				Values:     []float64{0, 1, 2},
 			},
 		}
-		rollupResultCacheV.Put(nil, ec, fe, window, tss)
-		tss, newStart := rollupResultCacheV.Get(nil, ec, fe, window)
+		rollupResultCacheV.PutSeries(nil, ec, fe, window, tss)
+		tss, newStart := rollupResultCacheV.GetSeries(nil, ec, fe, window)
 		if newStart != 1000 {
 			t.Fatalf("unexpected newStart; got %d; want %d", newStart, 1000)
 		}
@@ -156,8 +162,8 @@ func TestRollupResultCache(t *testing.T) {
 				Values:     []float64{0, 1, 2},
 			},
 		}
-		rollupResultCacheV.Put(nil, ec, fe, window, tss)
-		tss, newStart := rollupResultCacheV.Get(nil, ec, fe, window)
+		rollupResultCacheV.PutSeries(nil, ec, fe, window, tss)
+		tss, newStart := rollupResultCacheV.GetSeries(nil, ec, fe, window)
 		if newStart != 1000 {
 			t.Fatalf("unexpected newStart; got %d; want %d", newStart, 1000)
 		}
@@ -175,8 +181,8 @@ func TestRollupResultCache(t *testing.T) {
 				Values:     []float64{0, 1, 2},
 			},
 		}
-		rollupResultCacheV.Put(nil, ec, fe, window, tss)
-		tss, newStart := rollupResultCacheV.Get(nil, ec, fe, window)
+		rollupResultCacheV.PutSeries(nil, ec, fe, window, tss)
+		tss, newStart := rollupResultCacheV.GetSeries(nil, ec, fe, window)
 		if newStart != 1000 {
 			t.Fatalf("unexpected newStart; got %d; want %d", newStart, 1000)
 		}
@@ -194,8 +200,8 @@ func TestRollupResultCache(t *testing.T) {
 				Values:     []float64{0, 1, 2, 3, 4, 5, 6, 7},
 			},
 		}
-		rollupResultCacheV.Put(nil, ec, fe, window, tss)
-		tss, newStart := rollupResultCacheV.Get(nil, ec, fe, window)
+		rollupResultCacheV.PutSeries(nil, ec, fe, window, tss)
+		tss, newStart := rollupResultCacheV.GetSeries(nil, ec, fe, window)
 		if newStart != 2200 {
 			t.Fatalf("unexpected newStart; got %d; want %d", newStart, 2200)
 		}
@@ -217,8 +223,8 @@ func TestRollupResultCache(t *testing.T) {
 				Values:     []float64{1, 2, 3, 4, 5, 6},
 			},
 		}
-		rollupResultCacheV.Put(nil, ec, fe, window, tss)
-		tss, newStart := rollupResultCacheV.Get(nil, ec, fe, window)
+		rollupResultCacheV.PutSeries(nil, ec, fe, window, tss)
+		tss, newStart := rollupResultCacheV.GetSeries(nil, ec, fe, window)
 		if newStart != 2200 {
 			t.Fatalf("unexpected newStart; got %d; want %d", newStart, 2200)
 		}
@@ -240,16 +246,40 @@ func TestRollupResultCache(t *testing.T) {
 				Timestamps: []int64{1000, 1200, 1400, 1600, 1800, 2000},
 				Values:     []float64{1, 2, 3, 4, 5, 6},
 			}
+			ts.MetricName.MetricGroup = []byte(fmt.Sprintf("metric %d", i))
 			tss = append(tss, ts)
 		}
-		rollupResultCacheV.Put(nil, ec, fe, window, tss)
-		tssResult, newStart := rollupResultCacheV.Get(nil, ec, fe, window)
+		rollupResultCacheV.PutSeries(nil, ec, fe, window, tss)
+		tssResult, newStart := rollupResultCacheV.GetSeries(nil, ec, fe, window)
 		if newStart != 2200 {
 			t.Fatalf("unexpected newStart; got %d; want %d", newStart, 2200)
 		}
 		testTimeseriesEqual(t, tssResult, tss)
 	})
 
+	// Store series with identical naming (they shouldn't be stored)
+	t.Run("duplicate-series", func(t *testing.T) {
+		ResetRollupResultCache()
+		tss := []*timeseries{
+			{
+				Timestamps: []int64{800, 1000, 1200},
+				Values:     []float64{0, 1, 2},
+			},
+			{
+				Timestamps: []int64{800, 1000, 1200},
+				Values:     []float64{0, 1, 2},
+			},
+		}
+		rollupResultCacheV.PutSeries(nil, ec, fe, window, tss)
+		tssResult, newStart := rollupResultCacheV.GetSeries(nil, ec, fe, window)
+		if newStart != ec.Start {
+			t.Fatalf("unexpected newStart; got %d; want %d", newStart, ec.Start)
+		}
+		if len(tssResult) != 0 {
+			t.Fatalf("unexpected non-empty series returned")
+		}
+	})
+
 	// Store multiple time series
 	t.Run("multi-timeseries", func(t *testing.T) {
 		ResetRollupResultCache()
@@ -271,10 +301,10 @@ func TestRollupResultCache(t *testing.T) {
 				Values:     []float64{0, 1, 2},
 			},
 		}
-		rollupResultCacheV.Put(nil, ec, fe, window, tss1)
-		rollupResultCacheV.Put(nil, ec, fe, window, tss2)
-		rollupResultCacheV.Put(nil, ec, fe, window, tss3)
-		tss, newStart := rollupResultCacheV.Get(nil, ec, fe, window)
+		rollupResultCacheV.PutSeries(nil, ec, fe, window, tss1)
+		rollupResultCacheV.PutSeries(nil, ec, fe, window, tss2)
+		rollupResultCacheV.PutSeries(nil, ec, fe, window, tss3)
+		tss, newStart := rollupResultCacheV.GetSeries(nil, ec, fe, window)
 		if newStart != 1400 {
 			t.Fatalf("unexpected newStart; got %d; want %d", newStart, 1400)
 		}
@@ -289,11 +319,12 @@ func TestRollupResultCache(t *testing.T) {
 
 }
 
-func TestMergeTimeseries(t *testing.T) {
+func TestMergeSeries(t *testing.T) {
 	ec := &EvalConfig{
-		Start: 1000,
-		End:   2000,
-		Step:  200,
+		Start:              1000,
+		End:                2000,
+		Step:               200,
+		MaxPointsPerSeries: 1e4,
 	}
 	bStart := int64(1400)
 
@@ -305,7 +336,10 @@ func TestMergeTimeseries(t *testing.T) {
 				Values:     []float64{1, 2, 3, 4, 5, 6},
 			},
 		}
-		tss := mergeTimeseries(a, b, 1000, ec)
+		tss, ok := mergeSeries(nil, a, b, 1000, ec)
+		if !ok {
+			t.Fatalf("unexpected failure to merge series")
+		}
 		tssExpected := []*timeseries{
 			{
 				Timestamps: []int64{1000, 1200, 1400, 1600, 1800, 2000},
@@ -322,7 +356,10 @@ func TestMergeTimeseries(t *testing.T) {
 				Values:     []float64{3, 4, 5, 6},
 			},
 		}
-		tss := mergeTimeseries(a, b, bStart, ec)
+		tss, ok := mergeSeries(nil, a, b, bStart, ec)
+		if !ok {
+			t.Fatalf("unexpected failure to merge series")
+		}
 		tssExpected := []*timeseries{
 			{
 				Timestamps: []int64{1000, 1200, 1400, 1600, 1800, 2000},
@@ -339,7 +376,10 @@ func TestMergeTimeseries(t *testing.T) {
 			},
 		}
 		b := []*timeseries{}
-		tss := mergeTimeseries(a, b, bStart, ec)
+		tss, ok := mergeSeries(nil, a, b, bStart, ec)
+		if !ok {
+			t.Fatalf("unexpected failure to merge series")
+		}
 		tssExpected := []*timeseries{
 			{
 				Timestamps: []int64{1000, 1200, 1400, 1600, 1800, 2000},
@@ -361,7 +401,10 @@ func TestMergeTimeseries(t *testing.T) {
 				Values:     []float64{3, 4, 5, 6},
 			},
 		}
-		tss := mergeTimeseries(a, b, bStart, ec)
+		tss, ok := mergeSeries(nil, a, b, bStart, ec)
+		if !ok {
+			t.Fatalf("unexpected failure to merge series")
+		}
 		tssExpected := []*timeseries{
 			{
 				Timestamps: []int64{1000, 1200, 1400, 1600, 1800, 2000},
@@ -385,7 +428,10 @@ func TestMergeTimeseries(t *testing.T) {
 			},
 		}
 		b[0].MetricName.MetricGroup = []byte("foo")
-		tss := mergeTimeseries(a, b, bStart, ec)
+		tss, ok := mergeSeries(nil, a, b, bStart, ec)
+		if !ok {
+			t.Fatalf("unexpected failure to merge series")
+		}
 		tssExpected := []*timeseries{
 			{
 				MetricName: storage.MetricName{
@@ -404,6 +450,52 @@ func TestMergeTimeseries(t *testing.T) {
 		}
 		testTimeseriesEqual(t, tss, tssExpected)
 	})
+	t.Run("duplicate-series-a", func(t *testing.T) {
+		a := []*timeseries{
+			{
+				Timestamps: []int64{1000, 1200},
+				Values:     []float64{2, 1},
+			},
+			{
+				Timestamps: []int64{1000, 1200},
+				Values:     []float64{3, 3},
+			},
+		}
+		b := []*timeseries{
+			{
+				Timestamps: []int64{1400, 1600, 1800, 2000},
+				Values:     []float64{3, 4, 5, 6},
+			},
+		}
+		tss, ok := mergeSeries(nil, a, b, bStart, ec)
+		if ok {
+			t.Fatalf("expecting failre to merge series")
+		}
+		testTimeseriesEqual(t, tss, nil)
+	})
+	t.Run("duplicate-series-b", func(t *testing.T) {
+		a := []*timeseries{
+			{
+				Timestamps: []int64{1000, 1200},
+				Values:     []float64{1, 2},
+			},
+		}
+		b := []*timeseries{
+			{
+				Timestamps: []int64{1400, 1600, 1800, 2000},
+				Values:     []float64{3, 4, 5, 6},
+			},
+			{
+				Timestamps: []int64{1400, 1600, 1800, 2000},
+				Values:     []float64{13, 14, 15, 16},
+			},
+		}
+		tss, ok := mergeSeries(nil, a, b, bStart, ec)
+		if ok {
+			t.Fatalf("expecting failre to merge series")
+		}
+		testTimeseriesEqual(t, tss, nil)
+	})
 }
 
 func testTimeseriesEqual(t *testing.T, tss, tssExpected []*timeseries) {
