@@ -29,10 +29,7 @@ func TestWriteRead(t *testing.T) {
 func testWriteRead(t *testing.T, nocache bool, testStr string) {
 	t.Helper()
 
-	w, err := Create("./nocache_test.txt", nocache)
-	if err != nil {
-		t.Fatalf("cannot create file: %s", err)
-	}
+	w := MustCreate("./nocache_test.txt", nocache)
 	defer func() {
 		_ = os.Remove("./nocache_test.txt")
 	}()
@@ -42,10 +39,7 @@ func testWriteRead(t *testing.T, nocache bool, testStr string) {
 	}
 	w.MustClose()
 
-	r, err := Open("./nocache_test.txt", nocache)
-	if err != nil {
-		t.Fatalf("cannot open file: %s", err)
-	}
+	r := MustOpen("./nocache_test.txt", nocache)
 	buf := make([]byte, len(testStr))
 	if _, err := io.ReadFull(r, buf); err != nil {
 		t.Fatalf("unexpected error when reading: %s", err)
