@@ -12,7 +12,6 @@ func TestBytesSetFailure(t *testing.T) {
 			t.Fatalf("expecting non-nil error in b.Set(%q)", value)
 		}
 	}
-	f("")
 	f("foobar")
 	f("5foobar")
 	f("aKB")
@@ -39,6 +38,7 @@ func TestBytesSetSuccess(t *testing.T) {
 			t.Fatalf("unexpected valueString; got %q; want %q", valueString, valueExpected)
 		}
 	}
+	f("", 0)
 	f("0", 0)
 	f("1", 1)
 	f("-1234", -1234)
@@ -47,8 +47,10 @@ func TestBytesSetSuccess(t *testing.T) {
 	f("1.5kib", 1.5*1024)
 	f("23MiB", 23*1024*1024)
 	f("0.25GiB", 0.25*1024*1024*1024)
+	f("1.25TiB", 1.25*1024*1024*1024*1024)
 	f("1KB", 1000)
 	f("1.5kb", 1.5*1000)
 	f("23MB", 23*1000*1000)
 	f("0.25GB", 0.25*1000*1000*1000)
+	f("1.25TB", 1.25*1000*1000*1000*1000)
 }
