@@ -1,5 +1,4 @@
 //go:build linux || darwin || freebsd || openbsd
-// +build linux darwin freebsd openbsd
 
 package fs
 
@@ -7,6 +6,7 @@ import (
 	"fmt"
 	"os"
 
+	"github.com/VictoriaMetrics/VictoriaMetrics/lib/fs/fsutil"
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/logger"
 	"golang.org/x/sys/unix"
 )
@@ -22,11 +22,13 @@ func mUnmap(data []byte) error {
 func mustSyncPath(path string) {
 	d, err := os.Open(path)
 	if err != nil {
-		logger.Panicf("FATAL: cannot open %q: %s", path, err)
+		logger.Panicf("FATAL: cannot open file for fsync: %s", err)
 	}
-	if err := d.Sync(); err != nil {
-		_ = d.Close()
-		logger.Panicf("FATAL: cannot flush %q to storage: %s", path, err)
+	if !fsutil.IsFsyncDisabled() {
+		if err := d.Sync(); err != nil {
+			_ = d.Close()
+			logger.Panicf("FATAL: cannot flush %q to storage: %s", path, err)
+		}
 	}
 	if err := d.Close(); err != nil {
 		logger.Panicf("FATAL: cannot close %q: %s", path, err)
@@ -45,15 +47,8 @@ func createFlockFile(flockFile string) (*os.File, error) {
 }
 
 func mustGetFreeSpace(path string) uint64 {
-	d, err := os.Open(path)
-	if err != nil {
-		logger.Panicf("FATAL: cannot determine free disk space on %q: %s", path, err)
-	}
-	defer MustClose(d)
-
-	fd := d.Fd()
 	var stat unix.Statfs_t
-	if err := unix.Fstatfs(int(fd), &stat); err != nil {
+	if err := unix.Statfs(path, &stat); err != nil {
 		logger.Panicf("FATAL: cannot determine free disk space on %q: %s", path, err)
 	}
 	return freeSpace(stat)
