@@ -2,7 +2,7 @@ package fs
 
 import (
 	"fmt"
-	"io/ioutil"
+	"os"
 	"testing"
 )
 
@@ -18,7 +18,7 @@ func testReaderAt(t *testing.T, bufSize int) {
 	path := "TestReaderAt"
 	const fileSize = 8 * 1024 * 1024
 	data := make([]byte, fileSize)
-	if err := ioutil.WriteFile(path, data, 0600); err != nil {
+	if err := os.WriteFile(path, data, 0600); err != nil {
 		t.Fatalf("cannot create %q: %s", path, err)
 	}
 	defer MustRemoveAll(path)
