@@ -10,13 +10,13 @@ import (
 
 func logAllFlags() {
 	Infof("build version: %s", buildinfo.Version)
-	Infof("command line flags")
+	Infof("command-line flags")
 	flag.Visit(func(f *flag.Flag) {
 		lname := strings.ToLower(f.Name)
 		value := f.Value.String()
 		if flagutil.IsSecretFlag(lname) {
 			value = "secret"
 		}
-		Infof("flag %q=%q", f.Name, value)
+		Infof("  -%s=%q", f.Name, value)
 	})
 }
