@@ -530,15 +530,11 @@ func (ddb *datadb) mustMergeParts(pws []*partWrapper, isFinal bool) {
 	srcSize := uint64(0)
 	srcRowsCount := uint64(0)
 	srcBlocksCount := uint64(0)
-	bloomValuesShardsCount := uint64(0)
 	for _, pw := range pws {
 		ph := &pw.p.ph
 		srcSize += ph.CompressedSizeBytes
 		srcRowsCount += ph.RowsCount
 		srcBlocksCount += ph.BlocksCount
-		if ph.BloomValuesFieldsCount > bloomValuesShardsCount {
-			bloomValuesShardsCount = ph.BloomValuesFieldsCount
-		}
 	}
 	bsw := getBlockStreamWriter()
 	var mpNew *inmemoryPart
@@ -547,7 +543,7 @@ func (ddb *datadb) mustMergeParts(pws []*partWrapper, isFinal bool) {
 		bsw.MustInitForInmemoryPart(mpNew)
 	} else {
 		nocache := dstPartType == partBig
-		bsw.MustInitForFilePart(dstPartPath, nocache, bloomValuesShardsCount)
+		bsw.MustInitForFilePart(dstPartPath, nocache)
 	}
 
 	// Merge source parts to destination part.
