@@ -1,10 +1,11 @@
 package mergeset
 
 import (
-	"sync"
+	"path/filepath"
 
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/bytesutil"
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/encoding"
+	"github.com/VictoriaMetrics/VictoriaMetrics/lib/fs"
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/logger"
 )
 
@@ -30,6 +31,28 @@ func (mp *inmemoryPart) Reset() {
 	mp.lensData.Reset()
 }
 
+// MustStoreToDisk stores mp to the given path on disk.
+func (mp *inmemoryPart) MustStoreToDisk(path string) {
+	fs.MustMkdirFailIfExist(path)
+
+	metaindexPath := filepath.Join(path, metaindexFilename)
+	fs.MustWriteSync(metaindexPath, mp.metaindexData.B)
+
+	indexPath := filepath.Join(path, indexFilename)
+	fs.MustWriteSync(indexPath, mp.indexData.B)
+
+	itemsPath := filepath.Join(path, itemsFilename)
+	fs.MustWriteSync(itemsPath, mp.itemsData.B)
+
+	lensPath := filepath.Join(path, lensFilename)
+	fs.MustWriteSync(lensPath, mp.lensData.B)
+
+	mp.ph.MustWriteMetadata(path)
+
+	fs.MustSyncPath(path)
+	// Do not sync parent directory - it must be synced by the caller.
+}
+
 // Init initializes mp from ib.
 func (mp *inmemoryPart) Init(ib *inmemoryBlock) {
 	mp.Reset()
@@ -62,14 +85,18 @@ func (mp *inmemoryPart) Init(ib *inmemoryBlock) {
 
 	bb := inmemoryPartBytePool.Get()
 	bb.B = mp.bh.Marshal(bb.B[:0])
-	mp.indexData.B = encoding.CompressZSTDLevel(mp.indexData.B[:0], bb.B, 0)
+	if len(bb.B) > 3*maxIndexBlockSize {
+		// marshaled blockHeader can exceed indexBlockSize when firstItem and commonPrefix sizes are close to indexBlockSize
+		logger.Panicf("BUG: too big index block: %d bytes; mustn't exceed %d bytes", len(bb.B), 3*maxIndexBlockSize)
+	}
+	mp.indexData.B = encoding.CompressZSTDLevel(mp.indexData.B[:0], bb.B, compressLevel)
 
 	mp.mr.firstItem = append(mp.mr.firstItem[:0], mp.bh.firstItem...)
 	mp.mr.blockHeadersCount = 1
 	mp.mr.indexBlockOffset = 0
 	mp.mr.indexBlockSize = uint32(len(mp.indexData.B))
 	bb.B = mp.mr.Marshal(bb.B[:0])
-	mp.metaindexData.B = encoding.CompressZSTDLevel(mp.metaindexData.B[:0], bb.B, 0)
+	mp.metaindexData.B = encoding.CompressZSTDLevel(mp.metaindexData.B[:0], bb.B, compressLevel)
 	inmemoryPartBytePool.Put(bb)
 }
 
@@ -78,30 +105,11 @@ var inmemoryPartBytePool bytesutil.ByteBufferPool
 // It is safe calling NewPart multiple times.
 // It is unsafe re-using mp while the returned part is in use.
 func (mp *inmemoryPart) NewPart() *part {
-	ph := mp.ph
 	size := mp.size()
-	p, err := newPart(&ph, "", size, mp.metaindexData.NewReader(), &mp.indexData, &mp.itemsData, &mp.lensData)
-	if err != nil {
-		logger.Panicf("BUG: cannot create a part from inmemoryPart: %s", err)
-	}
+	p := newPart(&mp.ph, "", size, mp.metaindexData.NewReader(), &mp.indexData, &mp.itemsData, &mp.lensData)
 	return p
 }
 
 func (mp *inmemoryPart) size() uint64 {
-	return uint64(len(mp.metaindexData.B) + len(mp.indexData.B) + len(mp.itemsData.B) + len(mp.lensData.B))
+	return uint64(cap(mp.metaindexData.B) + cap(mp.indexData.B) + cap(mp.itemsData.B) + cap(mp.lensData.B))
 }
-
-func getInmemoryPart() *inmemoryPart {
-	v := inmemoryPartPool.Get()
-	if v == nil {
-		return &inmemoryPart{}
-	}
-	return v.(*inmemoryPart)
-}
-
-func putInmemoryPart(mp *inmemoryPart) {
-	mp.Reset()
-	inmemoryPartPool.Put(mp)
-}
-
-var inmemoryPartPool sync.Pool
