@@ -27,7 +27,8 @@ func TestTableSearchSerial(t *testing.T) {
 	const itemsCount = 1e5
 
 	items := func() []string {
-		tb, items, err := newTestTable(path, itemsCount)
+		r := rand.New(rand.NewSource(1))
+		tb, items, err := newTestTable(r, path, itemsCount)
 		if err != nil {
 			t.Fatalf("cannot create test table: %s", err)
 		}
@@ -40,11 +41,8 @@ func TestTableSearchSerial(t *testing.T) {
 
 	func() {
 		// Re-open the table and verify the search works.
-		var isReadOnly uint32
-		tb, err := OpenTable(path, nil, nil, &isReadOnly)
-		if err != nil {
-			t.Fatalf("cannot open table: %s", err)
-		}
+		var isReadOnly atomic.Bool
+		tb := MustOpenTable(path, nil, nil, &isReadOnly)
 		defer tb.MustClose()
 		if err := testTableSearchSerial(tb, items); err != nil {
 			t.Fatalf("unexpected error: %s", err)
@@ -63,7 +61,8 @@ func TestTableSearchConcurrent(t *testing.T) {
 
 	const itemsCount = 1e5
 	items := func() []string {
-		tb, items, err := newTestTable(path, itemsCount)
+		r := rand.New(rand.NewSource(2))
+		tb, items, err := newTestTable(r, path, itemsCount)
 		if err != nil {
 			t.Fatalf("cannot create test table: %s", err)
 		}
@@ -76,11 +75,8 @@ func TestTableSearchConcurrent(t *testing.T) {
 
 	// Re-open the table and verify the search works.
 	func() {
-		var isReadOnly uint32
-		tb, err := OpenTable(path, nil, nil, &isReadOnly)
-		if err != nil {
-			t.Fatalf("cannot open table: %s", err)
-		}
+		var isReadOnly atomic.Bool
+		tb := MustOpenTable(path, nil, nil, &isReadOnly)
 		defer tb.MustClose()
 		if err := testTableSearchConcurrent(tb, items); err != nil {
 			t.Fatalf("unexpected error: %s", err)
@@ -111,7 +107,7 @@ func testTableSearchConcurrent(tb *Table, items []string) error {
 
 func testTableSearchSerial(tb *Table, items []string) error {
 	var ts TableSearch
-	ts.Init(tb)
+	ts.Init(tb, false)
 	for _, key := range []string{
 		"",
 		"123",
@@ -148,26 +144,21 @@ func testTableSearchSerial(tb *Table, items []string) error {
 	return nil
 }
 
-func newTestTable(path string, itemsCount int) (*Table, []string, error) {
-	var flushes uint64
+func newTestTable(r *rand.Rand, path string, itemsCount int) (*Table, []string, error) {
+	var flushes atomic.Uint64
 	flushCallback := func() {
-		atomic.AddUint64(&flushes, 1)
-	}
-	var isReadOnly uint32
-	tb, err := OpenTable(path, flushCallback, nil, &isReadOnly)
-	if err != nil {
-		return nil, nil, fmt.Errorf("cannot open table: %w", err)
+		flushes.Add(1)
 	}
+	var isReadOnly atomic.Bool
+	tb := MustOpenTable(path, flushCallback, nil, &isReadOnly)
 	items := make([]string, itemsCount)
 	for i := 0; i < itemsCount; i++ {
-		item := fmt.Sprintf("%d:%d", rand.Intn(1e9), i)
-		if err := tb.AddItems([][]byte{[]byte(item)}); err != nil {
-			return nil, nil, fmt.Errorf("cannot add item: %w", err)
-		}
+		item := fmt.Sprintf("%d:%d", r.Intn(1e9), i)
+		tb.AddItems([][]byte{[]byte(item)})
 		items[i] = item
 	}
 	tb.DebugFlush()
-	if itemsCount > 0 && atomic.LoadUint64(&flushes) == 0 {
+	if itemsCount > 0 && flushes.Load() == 0 {
 		return nil, nil, fmt.Errorf("unexpeted zero flushes for itemsCount=%d", itemsCount)
 	}
 
