@@ -3,12 +3,11 @@ package mergeset
 import (
 	"bytes"
 	"fmt"
+	"math/rand"
 	"os"
 	"sync"
 	"sync/atomic"
 	"testing"
-
-	"github.com/VictoriaMetrics/VictoriaMetrics/lib/logger"
 )
 
 func TestTableOpenClose(t *testing.T) {
@@ -21,49 +20,35 @@ func TestTableOpenClose(t *testing.T) {
 	}()
 
 	// Create a new table
-	var isReadOnly uint32
-	tb, err := OpenTable(path, nil, nil, &isReadOnly)
-	if err != nil {
-		t.Fatalf("cannot create new table: %s", err)
-	}
+	var isReadOnly atomic.Bool
+	tb := MustOpenTable(path, nil, nil, &isReadOnly)
 
 	// Close it
 	tb.MustClose()
 
 	// Re-open created table multiple times.
-	for i := 0; i < 10; i++ {
-		tb, err := OpenTable(path, nil, nil, &isReadOnly)
-		if err != nil {
-			t.Fatalf("cannot open created table: %s", err)
-		}
+	for i := 0; i < 4; i++ {
+		tb := MustOpenTable(path, nil, nil, &isReadOnly)
 		tb.MustClose()
 	}
 }
 
-func TestTableOpenMultipleTimes(t *testing.T) {
-	const path = "TestTableOpenMultipleTimes"
-	defer func() {
-		_ = os.RemoveAll(path)
-	}()
-
-	var isReadOnly uint32
-	tb1, err := OpenTable(path, nil, nil, &isReadOnly)
-	if err != nil {
-		t.Fatalf("cannot open table: %s", err)
+func TestTableAddItemsTooLongItem(t *testing.T) {
+	const path = "TestTableAddItemsTooLongItem"
+	if err := os.RemoveAll(path); err != nil {
+		t.Fatalf("cannot remove %q: %s", path, err)
 	}
-	defer tb1.MustClose()
 
-	for i := 0; i < 10; i++ {
-		tb2, err := OpenTable(path, nil, nil, &isReadOnly)
-		if err == nil {
-			tb2.MustClose()
-			t.Fatalf("expecting non-nil error when opening already opened table")
-		}
-	}
+	var isReadOnly atomic.Bool
+	tb := MustOpenTable(path, nil, nil, &isReadOnly)
+	tb.AddItems([][]byte{make([]byte, maxInmemoryBlockSize+1)})
+	tb.MustClose()
+	_ = os.RemoveAll(path)
 }
 
-func TestTableAddItemSerial(t *testing.T) {
-	const path = "TestTableAddItemSerial"
+func TestTableAddItemsSerial(t *testing.T) {
+	r := rand.New(rand.NewSource(1))
+	const path = "TestTableAddItemsSerial"
 	if err := os.RemoveAll(path); err != nil {
 		t.Fatalf("cannot remove %q: %s", path, err)
 	}
@@ -71,58 +56,50 @@ func TestTableAddItemSerial(t *testing.T) {
 		_ = os.RemoveAll(path)
 	}()
 
-	var flushes uint64
+	var flushes atomic.Uint64
 	flushCallback := func() {
-		atomic.AddUint64(&flushes, 1)
-	}
-	var isReadOnly uint32
-	tb, err := OpenTable(path, flushCallback, nil, &isReadOnly)
-	if err != nil {
-		t.Fatalf("cannot open %q: %s", path, err)
+		flushes.Add(1)
 	}
+	var isReadOnly atomic.Bool
+	tb := MustOpenTable(path, flushCallback, nil, &isReadOnly)
 
-	const itemsCount = 1e5
-	testAddItemsSerial(tb, itemsCount)
+	const itemsCount = 10e3
+	testAddItemsSerial(r, tb, itemsCount)
 
 	// Verify items count after pending items flush.
 	tb.DebugFlush()
-	if atomic.LoadUint64(&flushes) == 0 {
+	if flushes.Load() == 0 {
 		t.Fatalf("unexpected zero flushes")
 	}
 
 	var m TableMetrics
 	tb.UpdateMetrics(&m)
-	if m.ItemsCount != itemsCount {
-		t.Fatalf("unexpected itemsCount; got %d; want %v", m.ItemsCount, itemsCount)
+	if n := m.TotalItemsCount(); n != itemsCount {
+		t.Fatalf("unexpected itemsCount; got %d; want %v", n, itemsCount)
 	}
 
 	tb.MustClose()
 
-	// Re-open the table and make sure ItemsCount remains the same.
+	// Re-open the table and make sure itemsCount remains the same.
 	testReopenTable(t, path, itemsCount)
 
 	// Add more items in order to verify merge between inmemory parts and file-based parts.
-	tb, err = OpenTable(path, nil, nil, &isReadOnly)
-	if err != nil {
-		t.Fatalf("cannot open %q: %s", path, err)
-	}
+	tb = MustOpenTable(path, nil, nil, &isReadOnly)
 	const moreItemsCount = itemsCount * 3
-	testAddItemsSerial(tb, moreItemsCount)
+	testAddItemsSerial(r, tb, moreItemsCount)
 	tb.MustClose()
 
-	// Re-open the table and verify ItemsCount again.
+	// Re-open the table and verify itemsCount again.
 	testReopenTable(t, path, itemsCount+moreItemsCount)
 }
 
-func testAddItemsSerial(tb *Table, itemsCount int) {
+func testAddItemsSerial(r *rand.Rand, tb *Table, itemsCount int) {
 	for i := 0; i < itemsCount; i++ {
-		item := getRandomBytes()
+		item := getRandomBytes(r)
 		if len(item) > maxInmemoryBlockSize {
 			item = item[:maxInmemoryBlockSize]
 		}
-		if err := tb.AddItems([][]byte{item}); err != nil {
-			logger.Panicf("BUG: cannot add item to table: %s", err)
-		}
+		tb.AddItems([][]byte{item})
 	}
 }
 
@@ -131,26 +108,21 @@ func TestTableCreateSnapshotAt(t *testing.T) {
 	if err := os.RemoveAll(path); err != nil {
 		t.Fatalf("cannot remove %q: %s", path, err)
 	}
-	defer func() {
-		_ = os.RemoveAll(path)
-	}()
 
-	var isReadOnly uint32
-	tb, err := OpenTable(path, nil, nil, &isReadOnly)
-	if err != nil {
-		t.Fatalf("cannot open %q: %s", path, err)
-	}
-	defer tb.MustClose()
+	var isReadOnly atomic.Bool
+	tb := MustOpenTable(path, nil, nil, &isReadOnly)
 
 	// Write a lot of items into the table, so background merges would start.
 	const itemsCount = 3e5
 	for i := 0; i < itemsCount; i++ {
 		item := []byte(fmt.Sprintf("item %d", i))
-		if err := tb.AddItems([][]byte{item}); err != nil {
-			t.Fatalf("cannot add item to table: %s", err)
-		}
+		tb.AddItems([][]byte{item})
 	}
-	tb.DebugFlush()
+
+	// Close and open the table in order to flush all the data to disk before creating snapshots.
+	// See https://github.com/VictoriaMetrics/VictoriaMetrics/issues/4272#issuecomment-1550221840
+	tb.MustClose()
+	tb = MustOpenTable(path, nil, nil, &isReadOnly)
 
 	// Create multiple snapshots.
 	snapshot1 := path + "-test-snapshot1"
@@ -161,30 +133,15 @@ func TestTableCreateSnapshotAt(t *testing.T) {
 	if err := tb.CreateSnapshotAt(snapshot2); err != nil {
 		t.Fatalf("cannot create snapshot2: %s", err)
 	}
-	defer func() {
-		_ = os.RemoveAll(snapshot1)
-		_ = os.RemoveAll(snapshot2)
-	}()
 
 	// Verify snapshots contain all the data.
-	tb1, err := OpenTable(snapshot1, nil, nil, &isReadOnly)
-	if err != nil {
-		t.Fatalf("cannot open %q: %s", path, err)
-	}
-	defer tb1.MustClose()
-
-	tb2, err := OpenTable(snapshot2, nil, nil, &isReadOnly)
-	if err != nil {
-		t.Fatalf("cannot open %q: %s", path, err)
-	}
-	defer tb2.MustClose()
+	tb1 := MustOpenTable(snapshot1, nil, nil, &isReadOnly)
+	tb2 := MustOpenTable(snapshot2, nil, nil, &isReadOnly)
 
 	var ts, ts1, ts2 TableSearch
-	ts.Init(tb)
-	ts1.Init(tb1)
-	defer ts1.MustClose()
-	ts2.Init(tb2)
-	defer ts2.MustClose()
+	ts.Init(tb, false)
+	ts1.Init(tb1, false)
+	ts2.Init(tb2, false)
 	for i := 0; i < itemsCount; i++ {
 		key := []byte(fmt.Sprintf("item %d", i))
 		if err := ts.FirstItemWithPrefix(key); err != nil {
@@ -206,6 +163,77 @@ func TestTableCreateSnapshotAt(t *testing.T) {
 			t.Fatalf("unexpected item found for key=%q in snapshot2; got %q", key, ts2.Item)
 		}
 	}
+	ts1.MustClose()
+	ts2.MustClose()
+
+	// Close and remove tables.
+	tb2.MustClose()
+	tb1.MustClose()
+	tb.MustClose()
+
+	_ = os.RemoveAll(snapshot2)
+	_ = os.RemoveAll(snapshot1)
+	_ = os.RemoveAll(path)
+}
+
+func TestTableAddItemsConcurrentStress(t *testing.T) {
+	const path = "TestTableAddItemsConcurrentStress"
+	if err := os.RemoveAll(path); err != nil {
+		t.Fatalf("cannot remove %q: %s", path, err)
+	}
+	defer func() {
+		_ = os.RemoveAll(path)
+	}()
+
+	rawItemsShardsPerTableOrig := rawItemsShardsPerTable
+	maxBlocksPerShardOrig := maxBlocksPerShard
+	rawItemsShardsPerTable = 10
+	maxBlocksPerShard = 3
+	defer func() {
+		rawItemsShardsPerTable = rawItemsShardsPerTableOrig
+		maxBlocksPerShard = maxBlocksPerShardOrig
+	}()
+
+	var flushes atomic.Uint64
+	flushCallback := func() {
+		flushes.Add(1)
+	}
+	prepareBlock := func(data []byte, items []Item) ([]byte, []Item) {
+		return data, items
+	}
+
+	blocksNeeded := rawItemsShardsPerTable * maxBlocksPerShard * 10
+	testAddItems := func(tb *Table) {
+		itemsBatch := make([][]byte, 0)
+
+		for j := 0; j < blocksNeeded; j++ {
+			item := bytes.Repeat([]byte{byte(j)}, maxInmemoryBlockSize-10)
+			itemsBatch = append(itemsBatch, item)
+		}
+		tb.AddItems(itemsBatch)
+	}
+
+	var isReadOnly atomic.Bool
+	tb := MustOpenTable(path, flushCallback, prepareBlock, &isReadOnly)
+
+	testAddItems(tb)
+
+	// Verify items count after pending items flush.
+	tb.DebugFlush()
+	if flushes.Load() == 0 {
+		t.Fatalf("unexpected zero flushes")
+	}
+
+	var m TableMetrics
+	tb.UpdateMetrics(&m)
+	if n := m.TotalItemsCount(); n != uint64(blocksNeeded) {
+		t.Fatalf("unexpected itemsCount; got %d; want %v", n, blocksNeeded)
+	}
+
+	tb.MustClose()
+
+	// Re-open the table and make sure itemsCount remains the same.
+	testReopenTable(t, path, blocksNeeded)
 }
 
 func TestTableAddItemsConcurrent(t *testing.T) {
@@ -217,55 +245,43 @@ func TestTableAddItemsConcurrent(t *testing.T) {
 		_ = os.RemoveAll(path)
 	}()
 
-	var flushes uint64
+	var flushes atomic.Uint64
 	flushCallback := func() {
-		atomic.AddUint64(&flushes, 1)
+		flushes.Add(1)
 	}
-	var itemsMerged uint64
 	prepareBlock := func(data []byte, items []Item) ([]byte, []Item) {
-		atomic.AddUint64(&itemsMerged, uint64(len(items)))
 		return data, items
 	}
-	var isReadOnly uint32
-	tb, err := OpenTable(path, flushCallback, prepareBlock, &isReadOnly)
-	if err != nil {
-		t.Fatalf("cannot open %q: %s", path, err)
-	}
+	var isReadOnly atomic.Bool
+	tb := MustOpenTable(path, flushCallback, prepareBlock, &isReadOnly)
 
-	const itemsCount = 1e5
+	const itemsCount = 10e3
 	testAddItemsConcurrent(tb, itemsCount)
 
 	// Verify items count after pending items flush.
 	tb.DebugFlush()
-	if atomic.LoadUint64(&flushes) == 0 {
+	if flushes.Load() == 0 {
 		t.Fatalf("unexpected zero flushes")
 	}
-	n := atomic.LoadUint64(&itemsMerged)
-	if n < itemsCount {
-		t.Fatalf("too low number of items merged; got %v; must be at least %v", n, itemsCount)
-	}
 
 	var m TableMetrics
 	tb.UpdateMetrics(&m)
-	if m.ItemsCount != itemsCount {
-		t.Fatalf("unexpected itemsCount; got %d; want %v", m.ItemsCount, itemsCount)
+	if n := m.TotalItemsCount(); n != itemsCount {
+		t.Fatalf("unexpected itemsCount; got %d; want %v", n, itemsCount)
 	}
 
 	tb.MustClose()
 
-	// Re-open the table and make sure ItemsCount remains the same.
+	// Re-open the table and make sure itemsCount remains the same.
 	testReopenTable(t, path, itemsCount)
 
 	// Add more items in order to verify merge between inmemory parts and file-based parts.
-	tb, err = OpenTable(path, nil, nil, &isReadOnly)
-	if err != nil {
-		t.Fatalf("cannot open %q: %s", path, err)
-	}
+	tb = MustOpenTable(path, nil, nil, &isReadOnly)
 	const moreItemsCount = itemsCount * 3
 	testAddItemsConcurrent(tb, moreItemsCount)
 	tb.MustClose()
 
-	// Re-open the table and verify ItemsCount again.
+	// Re-open the table and verify itemsCount again.
 	testReopenTable(t, path, itemsCount+moreItemsCount)
 }
 
@@ -275,18 +291,17 @@ func testAddItemsConcurrent(tb *Table, itemsCount int) {
 	var wg sync.WaitGroup
 	for i := 0; i < goroutinesCount; i++ {
 		wg.Add(1)
-		go func() {
+		go func(n int) {
 			defer wg.Done()
+			r := rand.New(rand.NewSource(int64(n)))
 			for range workCh {
-				item := getRandomBytes()
+				item := getRandomBytes(r)
 				if len(item) > maxInmemoryBlockSize {
 					item = item[:maxInmemoryBlockSize]
 				}
-				if err := tb.AddItems([][]byte{item}); err != nil {
-					logger.Panicf("BUG: cannot add item to table: %s", err)
-				}
+				tb.AddItems([][]byte{item})
 			}
-		}()
+		}(i)
 	}
 	for i := 0; i < itemsCount; i++ {
 		workCh <- i
@@ -299,15 +314,12 @@ func testReopenTable(t *testing.T, path string, itemsCount int) {
 	t.Helper()
 
 	for i := 0; i < 10; i++ {
-		var isReadOnly uint32
-		tb, err := OpenTable(path, nil, nil, &isReadOnly)
-		if err != nil {
-			t.Fatalf("cannot re-open %q: %s", path, err)
-		}
+		var isReadOnly atomic.Bool
+		tb := MustOpenTable(path, nil, nil, &isReadOnly)
 		var m TableMetrics
 		tb.UpdateMetrics(&m)
-		if m.ItemsCount != uint64(itemsCount) {
-			t.Fatalf("unexpected itemsCount after re-opening; got %d; want %v", m.ItemsCount, itemsCount)
+		if n := m.TotalItemsCount(); n != uint64(itemsCount) {
+			t.Fatalf("unexpected itemsCount after re-opening; got %d; want %v", n, itemsCount)
 		}
 		tb.MustClose()
 	}
