@@ -4,8 +4,8 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
+	"path/filepath"
 	"regexp"
 	"strconv"
 	"time"
@@ -22,7 +22,8 @@ import (
 // MaxBlockSize is the maximum size of the block persistent queue can work with.
 const MaxBlockSize = 32 * 1024 * 1024
 
-const defaultChunkFileSize = (MaxBlockSize + 8) * 16
+// DefaultChunkFileSize represents default chunk file size
+const DefaultChunkFileSize = (MaxBlockSize + 8) * 16
 
 var chunkFileNameRegex = regexp.MustCompile("^[0-9A-F]{16}$")
 
@@ -93,17 +94,11 @@ func (q *queue) mustResetFiles() {
 	q.readerLocalOffset = 0
 
 	q.writerPath = q.chunkFilePath(q.writerOffset)
-	w, err := filestream.Create(q.writerPath, false)
-	if err != nil {
-		logger.Panicf("FATAL: cannot create chunk file %q: %s", q.writerPath, err)
-	}
+	w := filestream.MustCreate(q.writerPath, false)
 	q.writer = w
 
 	q.readerPath = q.writerPath
-	r, err := filestream.Open(q.readerPath, true)
-	if err != nil {
-		logger.Panicf("FATAL: cannot open chunk file %q: %s", q.readerPath, err)
-	}
+	r := filestream.MustOpen(q.readerPath, true)
 	q.reader = r
 
 	if err := q.flushMetainfo(); err != nil {
@@ -128,7 +123,7 @@ func mustOpen(path, name string, maxPendingBytes int64) *queue {
 	if maxPendingBytes < 0 {
 		maxPendingBytes = 0
 	}
-	return mustOpenInternal(path, name, defaultChunkFileSize, MaxBlockSize, uint64(maxPendingBytes))
+	return mustOpenInternal(path, name, DefaultChunkFileSize, MaxBlockSize, uint64(maxPendingBytes))
 }
 
 func mustOpenInternal(path, name string, chunkFileSize, maxBlockSize, maxPendingBytes uint64) *queue {
@@ -150,14 +145,6 @@ func mustOpenInternal(path, name string, chunkFileSize, maxBlockSize, maxPending
 	return q
 }
 
-func mustCreateFlockFile(path string) *os.File {
-	f, err := fs.CreateFlockFile(path)
-	if err != nil {
-		logger.Panicf("FATAL: %s", err)
-	}
-	return f
-}
-
 func tryOpeningQueue(path, name string, chunkFileSize, maxBlockSize, maxPendingBytes uint64) (*queue, error) {
 	// Protect from concurrent opens.
 	var q queue
@@ -183,14 +170,12 @@ func tryOpeningQueue(path, name string, chunkFileSize, maxBlockSize, maxPendingB
 		}
 	}
 
-	if err := fs.MkdirAllIfNotExist(path); err != nil {
-		return nil, fmt.Errorf("cannot create directory %q: %w", path, err)
-	}
-	q.flockF = mustCreateFlockFile(path)
+	fs.MustMkdirIfNotExist(path)
+	q.flockF = fs.MustCreateFlockFile(path)
 	mustCloseFlockF := true
 	defer func() {
 		if mustCloseFlockF {
-			_ = q.flockF.Close()
+			fs.MustClose(q.flockF)
 		}
 	}()
 
@@ -203,7 +188,9 @@ func tryOpeningQueue(path, name string, chunkFileSize, maxBlockSize, maxPendingB
 		}
 
 		// path contents is broken or missing. Re-create it from scratch.
+		fs.MustClose(q.flockF)
 		fs.RemoveDirContents(path)
+		q.flockF = fs.MustCreateFlockFile(path)
 		mi.Reset()
 		mi.Name = q.name
 		if err := mi.WriteToFile(metainfoPath); err != nil {
@@ -212,31 +199,26 @@ func tryOpeningQueue(path, name string, chunkFileSize, maxBlockSize, maxPendingB
 
 		// Create initial chunk file.
 		filepath := q.chunkFilePath(0)
-		if err := fs.WriteFileAtomically(filepath, nil); err != nil {
-			return nil, fmt.Errorf("cannot create %q: %w", filepath, err)
-		}
+		fs.MustWriteAtomic(filepath, nil, false)
 	}
 	if mi.Name != q.name {
 		return nil, fmt.Errorf("unexpected queue name; got %q; want %q", mi.Name, q.name)
 	}
 
 	// Locate reader and writer chunks in the path.
-	fis, err := ioutil.ReadDir(path)
-	if err != nil {
-		return nil, fmt.Errorf("cannot read contents of the directory %q: %w", path, err)
-	}
-	for _, fi := range fis {
-		fname := fi.Name()
-		filepath := path + "/" + fname
-		if fi.IsDir() {
+	des := fs.MustReadDir(path)
+	for _, de := range des {
+		fname := de.Name()
+		filepath := filepath.Join(path, fname)
+		if de.IsDir() {
 			logger.Errorf("skipping unknown directory %q", filepath)
 			continue
 		}
-		if fname == "metainfo.json" {
+		if fname == metainfoFilename {
 			// skip metainfo file
 			continue
 		}
-		if fname == "flock.lock" {
+		if fname == fs.FlockFilename {
 			// skip flock file
 			continue
 		}
@@ -350,18 +332,16 @@ func (q *queue) MustClose() {
 	}
 
 	// Close flockF
-	if err := q.flockF.Close(); err != nil {
-		logger.Panicf("FATAL: cannot close flock file: %s", err)
-	}
+	fs.MustClose(q.flockF)
 	q.flockF = nil
 }
 
 func (q *queue) chunkFilePath(offset uint64) string {
-	return fmt.Sprintf("%s/%016X", q.dir, offset)
+	return filepath.Join(q.dir, fmt.Sprintf("%016X", offset))
 }
 
 func (q *queue) metainfoPath() string {
-	return q.dir + "/metainfo.json"
+	return filepath.Join(q.dir, metainfoFilename)
 }
 
 // MustWriteBlock writes block to q.
@@ -452,10 +432,7 @@ func (q *queue) nextChunkFileForWrite() error {
 	q.writerFlushedOffset = q.writerOffset
 	q.writerLocalOffset = 0
 	q.writerPath = q.chunkFilePath(q.writerOffset)
-	w, err := filestream.Create(q.writerPath, false)
-	if err != nil {
-		return fmt.Errorf("cannot create chunk file %q: %w", q.writerPath, err)
-	}
+	w := filestream.MustCreate(q.writerPath, false)
 	q.writer = w
 	if err := q.flushMetainfo(); err != nil {
 		return fmt.Errorf("cannot flush metainfo: %w", err)
@@ -563,10 +540,7 @@ func (q *queue) nextChunkFileForRead() error {
 	}
 	q.readerLocalOffset = 0
 	q.readerPath = q.chunkFilePath(q.readerOffset)
-	r, err := filestream.Open(q.readerPath, true)
-	if err != nil {
-		return fmt.Errorf("cannot open chunk file %q: %w", q.readerPath, err)
-	}
+	r := filestream.MustOpen(q.readerPath, true)
 	q.reader = r
 	if err := q.flushMetainfo(); err != nil {
 		return fmt.Errorf("cannot flush metainfo: %w", err)
@@ -671,7 +645,7 @@ func (mi *metainfo) WriteToFile(path string) error {
 	if err != nil {
 		return fmt.Errorf("cannot marshal persistent queue metainfo %#v: %w", mi, err)
 	}
-	if err := ioutil.WriteFile(path, data, 0600); err != nil {
+	if err := os.WriteFile(path, data, 0600); err != nil {
 		return fmt.Errorf("cannot write persistent queue metainfo to %q: %w", path, err)
 	}
 	fs.MustSyncPath(path)
@@ -680,7 +654,7 @@ func (mi *metainfo) WriteToFile(path string) error {
 
 func (mi *metainfo) ReadFromFile(path string) error {
 	mi.Reset()
-	data, err := ioutil.ReadFile(path)
+	data, err := os.ReadFile(path)
 	if err != nil {
 		if os.IsNotExist(err) {
 			return err
