@@ -1,5 +1,4 @@
 //go:build !windows
-// +build !windows
 
 package procutil
 
@@ -26,6 +25,10 @@ func WaitForSigterm() os.Signal {
 			// Prevent from the program stop on SIGHUP
 			continue
 		}
+		// Stop listening for SIGINT and SIGTERM signals,
+		// so the app could be interrupted by sending these signals again
+		// in the case if the caller doesn't finish the app gracefully.
+		signal.Stop(ch)
 		return sig
 	}
 }
