@@ -3,43 +3,53 @@ package promrelabel
 import (
 	"bytes"
 	"encoding/json"
-	"fmt"
 	"testing"
 
-	"github.com/VictoriaMetrics/VictoriaMetrics/lib/prompbmarshal"
-	"github.com/VictoriaMetrics/VictoriaMetrics/lib/protoparser/prometheus"
+	"github.com/VictoriaMetrics/VictoriaMetrics/lib/promutils"
 	"gopkg.in/yaml.v2"
 )
 
 func TestIfExpressionParseFailure(t *testing.T) {
 	f := func(s string) {
 		t.Helper()
+
 		var ie IfExpression
 		if err := ie.Parse(s); err == nil {
 			t.Fatalf("expecting non-nil error when parsing %q", s)
 		}
 	}
+
 	f(`{`)
 	f(`{foo`)
 	f(`foo{`)
+	f(`foo{bar="a" or}`)
 }
 
 func TestIfExpressionParseSuccess(t *testing.T) {
 	f := func(s string) {
 		t.Helper()
+
 		var ie IfExpression
 		if err := ie.Parse(s); err != nil {
 			t.Fatalf("unexpected error: %s", err)
 		}
 	}
+
 	f(`foo`)
 	f(`{foo="bar"}`)
 	f(`foo{bar=~"baz", x!="y"}`)
+	f(`{a="b" or c="d",e="x"}`)
+	f(`foo{
+  bar="a",x="y" or
+  x="a",a="b" or
+  a="x"
+}`)
 }
 
 func TestIfExpressionMarshalUnmarshalJSON(t *testing.T) {
 	f := func(s, jsonExpected string) {
 		t.Helper()
+
 		var ie IfExpression
 		if err := ie.Parse(s); err != nil {
 			t.Fatalf("cannot parse ifExpression %q: %s", s, err)
@@ -63,22 +73,26 @@ func TestIfExpressionMarshalUnmarshalJSON(t *testing.T) {
 			t.Fatalf("unexpected data after unmarshal/marshal cycle;\ngot\n%s\nwant\n%s", data2, jsonExpected)
 		}
 	}
+
 	f("foo", `"foo"`)
 	f(`{foo="bar",baz=~"x.*"}`, `"{foo=\"bar\",baz=~\"x.*\"}"`)
+	f(`{a="b" or c="d",x="z"}`, `"{a=\"b\" or c=\"d\",x=\"z\"}"`)
 }
 
 func TestIfExpressionUnmarshalFailure(t *testing.T) {
 	f := func(s string) {
 		t.Helper()
+
 		var ie IfExpression
 		err := yaml.UnmarshalStrict([]byte(s), &ie)
 		if err == nil {
 			t.Fatalf("expecting non-nil error")
 		}
 	}
+
 	f(`{`)
 	f(`{x:y}`)
-	f(`[]`)
+	f(`[1]`)
 	f(`"{"`)
 	f(`'{'`)
 	f(`foo{bar`)
@@ -97,6 +111,7 @@ func TestIfExpressionUnmarshalFailure(t *testing.T) {
 func TestIfExpressionUnmarshalSuccess(t *testing.T) {
 	f := func(s string) {
 		t.Helper()
+
 		var ie IfExpression
 		if err := yaml.UnmarshalStrict([]byte(s), &ie); err != nil {
 			t.Fatalf("unexpected error during unmarshal: %s", err)
@@ -110,31 +125,61 @@ func TestIfExpressionUnmarshalSuccess(t *testing.T) {
 			t.Fatalf("unexpected marshaled data;\ngot\n%s\nwant\n%s", b, s)
 		}
 	}
+
 	f(`'{}'`)
 	f(`foo`)
 	f(`foo{bar="baz"}`)
 	f(`'{a="b", c!="d", e=~"g", h!~"d"}'`)
 	f(`foo{bar="zs",a=~"b|c"}`)
+	f(`foo{z="y" or bar="zs",a=~"b|c"}`)
+	f(`- foo
+- bar{baz="abc"}`)
+}
+
+func TestIfExpressionString(t *testing.T) {
+	f := func(s, resultExpected string) {
+		t.Helper()
+
+		var ie IfExpression
+		if err := yaml.UnmarshalStrict([]byte(s), &ie); err != nil {
+			t.Fatalf("cannot unmarshal if expression: %s", err)
+		}
+		result := ie.String()
+		if result != resultExpected {
+			t.Fatalf("unexpected result\ngot\n%s\nwant\n%s", result, resultExpected)
+		}
+	}
+
+	// empty filters
+	f(`'{}'`, `{}`)
+
+	// multiple fiters
+	f(`foo{bar="baz",a=~"bc.+",d!="e",g!~".*qwe"}`, `foo{bar="baz",a=~"bc.+",d!="e",g!~".*qwe"}`)
+
+	// multiple if expressions
+	f(`- foo
+- bar{baz="abc"}`, `foo,bar{baz="abc"}`)
 }
 
 func TestIfExpressionMatch(t *testing.T) {
 	f := func(ifExpr, metricWithLabels string) {
 		t.Helper()
+
 		var ie IfExpression
 		if err := yaml.UnmarshalStrict([]byte(ifExpr), &ie); err != nil {
 			t.Fatalf("unexpected error during unmarshal: %s", err)
 		}
-		labels, err := parseMetricWithLabels(metricWithLabels)
-		if err != nil {
-			t.Fatalf("cannot parse %s: %s", metricWithLabels, err)
-		}
-		if !ie.Match(labels) {
+		labels := promutils.MustNewLabelsFromString(metricWithLabels)
+		if !ie.Match(labels.GetLabels()) {
 			t.Fatalf("unexpected mismatch of ifExpr=%s for %s", ifExpr, metricWithLabels)
 		}
 	}
+
 	f(`foo`, `foo`)
 	f(`foo`, `foo{bar="baz",a="b"}`)
 	f(`foo{bar="a"}`, `foo{bar="a"}`)
+	f(`foo{bar="a" or baz="x"}`, `foo{bar="a"}`)
+	f(`foo{baz="x" or bar="a"}`, `foo{bar="a"}`)
 	f(`foo{bar="a"}`, `foo{x="y",bar="a",baz="b"}`)
 	f(`'{a=~"x|abc",y!="z"}'`, `m{x="aa",a="abc"}`)
 	f(`'{a=~"x|abc",y!="z"}'`, `m{x="aa",a="abc",y="qwe"}`)
@@ -157,21 +202,21 @@ func TestIfExpressionMatch(t *testing.T) {
 func TestIfExpressionMismatch(t *testing.T) {
 	f := func(ifExpr, metricWithLabels string) {
 		t.Helper()
+
 		var ie IfExpression
 		if err := yaml.UnmarshalStrict([]byte(ifExpr), &ie); err != nil {
 			t.Fatalf("unexpected error during unmarshal: %s", err)
 		}
-		labels, err := parseMetricWithLabels(metricWithLabels)
-		if err != nil {
-			t.Fatalf("cannot parse %s: %s", metricWithLabels, err)
-		}
-		if ie.Match(labels) {
+		labels := promutils.MustNewLabelsFromString(metricWithLabels)
+		if ie.Match(labels.GetLabels()) {
 			t.Fatalf("unexpected match of ifExpr=%s for %s", ifExpr, metricWithLabels)
 		}
 	}
+
 	f(`foo`, `bar`)
 	f(`foo`, `a{foo="bar"}`)
 	f(`foo{bar="a"}`, `foo`)
+	f(`foo{bar="a" or baz="a"}`, `foo`)
 	f(`foo{bar="a"}`, `foo{bar="b"}`)
 	f(`foo{bar="a"}`, `foo{baz="b",a="b"}`)
 	f(`'{a=~"x|abc",y!="z"}'`, `m{x="aa",a="xabc"}`)
@@ -186,34 +231,3 @@ func TestIfExpressionMismatch(t *testing.T) {
 	f(`'{foo!~"bar|"}'`, `abc`)
 	f(`'{foo!~"bar|"}'`, `abc{foo="bar"}`)
 }
-
-func parseMetricWithLabels(metricWithLabels string) ([]prompbmarshal.Label, error) {
-	// add a value to metricWithLabels, so it could be parsed by prometheus protocol parser.
-	s := metricWithLabels + " 123"
-	var rows prometheus.Rows
-	var err error
-	rows.UnmarshalWithErrLogger(s, func(s string) {
-		err = fmt.Errorf("error during metric parse: %s", s)
-	})
-	if err != nil {
-		return nil, err
-	}
-	if len(rows.Rows) != 1 {
-		return nil, fmt.Errorf("unexpected number of rows parsed; got %d; want 1", len(rows.Rows))
-	}
-	r := rows.Rows[0]
-	var lfs []prompbmarshal.Label
-	if r.Metric != "" {
-		lfs = append(lfs, prompbmarshal.Label{
-			Name:  "__name__",
-			Value: r.Metric,
-		})
-	}
-	for _, tag := range r.Tags {
-		lfs = append(lfs, prompbmarshal.Label{
-			Name:  tag.Key,
-			Value: tag.Value,
-		})
-	}
-	return lfs, nil
-}
