@@ -9,6 +9,8 @@ import (
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/bytesutil"
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/logger"
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/prompbmarshal"
+	"github.com/VictoriaMetrics/VictoriaMetrics/lib/promutils"
+	"github.com/VictoriaMetrics/VictoriaMetrics/lib/regexutil"
 	"github.com/cespare/xxhash/v2"
 )
 
@@ -16,73 +18,107 @@ import (
 //
 // See https://prometheus.io/docs/prometheus/latest/configuration/configuration/#relabel_config
 type parsedRelabelConfig struct {
-	SourceLabels []string
-	Separator    string
-	TargetLabel  string
-	Regex        *regexp.Regexp
-	Modulus      uint64
-	Replacement  string
-	Action       string
-	If           *IfExpression
+	// ruleOriginal contains the original relabeling rule for the given prasedRelabelConfig.
+	ruleOriginal string
+
+	SourceLabels  []string
+	Separator     string
+	TargetLabel   string
+	RegexAnchored *regexp.Regexp
+	Modulus       uint64
+	Replacement   string
+	Action        string
+	If            *IfExpression
 
 	graphiteMatchTemplate *graphiteMatchTemplate
 	graphiteLabelRules    []graphiteLabelRule
 
-	regexOriginal                *regexp.Regexp
-	hasCaptureGroupInTargetLabel bool
-	hasCaptureGroupInReplacement bool
+	regex         *regexutil.PromRegex
+	regexOriginal *regexp.Regexp
+
+	hasCaptureGroupInTargetLabel   bool
+	hasCaptureGroupInReplacement   bool
+	hasLabelReferenceInReplacement bool
+
+	stringReplacer   *bytesutil.FastStringTransformer
+	submatchReplacer *bytesutil.FastStringTransformer
+}
+
+// DebugStep contains debug information about a single relabeling rule step
+type DebugStep struct {
+	// Rule contains string representation of the rule step
+	Rule string
+
+	// In contains the input labels before the execution of the rule step
+	In string
+
+	// Out contains the output labels after the execution of the rule step
+	Out string
+}
+
+// String returns human-readable representation for ds
+func (ds DebugStep) String() string {
+	return fmt.Sprintf("rule=%q, in=%s, out=%s", ds.Rule, ds.In, ds.Out)
 }
 
 // String returns human-readable representation for prc.
 func (prc *parsedRelabelConfig) String() string {
-	return fmt.Sprintf("SourceLabels=%s, Separator=%s, TargetLabel=%s, Regex=%s, Modulus=%d, Replacement=%s, Action=%s, If=%s, graphiteMatchTemplate=%s, graphiteLabelRules=%s",
-		prc.SourceLabels, prc.Separator, prc.TargetLabel, prc.Regex, prc.Modulus, prc.Replacement, prc.Action, prc.If, prc.graphiteMatchTemplate, prc.graphiteLabelRules)
+	return prc.ruleOriginal
+}
+
+// ApplyDebug applies pcs to labels in debug mode.
+//
+// It returns DebugStep list - one entry per each applied relabeling step.
+func (pcs *ParsedConfigs) ApplyDebug(labels []prompbmarshal.Label) ([]prompbmarshal.Label, []DebugStep) {
+	// Protect from overwriting labels between len(labels) and cap(labels) by limiting labels capacity to its length.
+	labels = labels[:len(labels):len(labels)]
+
+	inStr := LabelsToString(labels)
+	var dss []DebugStep
+	if pcs != nil {
+		for _, prc := range pcs.prcs {
+			labels = prc.apply(labels, 0)
+			outStr := LabelsToString(labels)
+			dss = append(dss, DebugStep{
+				Rule: prc.String(),
+				In:   inStr,
+				Out:  outStr,
+			})
+			inStr = outStr
+			if len(labels) == 0 {
+				// All the labels have been removed.
+				return labels, dss
+			}
+		}
+	}
+
+	labels = removeEmptyLabels(labels, 0)
+	outStr := LabelsToString(labels)
+	if outStr != inStr {
+		dss = append(dss, DebugStep{
+			Rule: "remove empty labels",
+			In:   inStr,
+			Out:  outStr,
+		})
+	}
+	return labels, dss
 }
 
 // Apply applies pcs to labels starting from the labelsOffset.
 //
-// If isFinalize is set, then FinalizeLabels is called on the labels[labelsOffset:].
-//
-// The returned labels at labels[labelsOffset:] are sorted.
-func (pcs *ParsedConfigs) Apply(labels []prompbmarshal.Label, labelsOffset int, isFinalize bool) []prompbmarshal.Label {
-	var inStr string
-	relabelDebug := false
+// This function may add additional labels after the len(labels), so make sure it doesn't corrupt in-use labels
+// stored between len(labels) and cap(labels).
+func (pcs *ParsedConfigs) Apply(labels []prompbmarshal.Label, labelsOffset int) []prompbmarshal.Label {
 	if pcs != nil {
-		relabelDebug = pcs.relabelDebug
-		if relabelDebug {
-			inStr = labelsToString(labels[labelsOffset:])
-		}
 		for _, prc := range pcs.prcs {
-			tmp := prc.apply(labels, labelsOffset)
-			if len(tmp) == labelsOffset {
+			labels = prc.apply(labels, labelsOffset)
+			if len(labels) == labelsOffset {
 				// All the labels have been removed.
-				if pcs.relabelDebug {
-					logger.Infof("\nRelabel  In: %s\nRelabel Out: DROPPED - all labels removed", inStr)
-				}
-				return tmp
+				return labels
 			}
-			labels = tmp
 		}
 	}
 	labels = removeEmptyLabels(labels, labelsOffset)
-	if isFinalize {
-		labels = FinalizeLabels(labels[:labelsOffset], labels[labelsOffset:])
-	}
-	SortLabels(labels[labelsOffset:])
-	if relabelDebug {
-		if len(labels) == labelsOffset {
-			logger.Infof("\nRelabel  In: %s\nRelabel Out: DROPPED - all labels removed", inStr)
-			return labels
-		}
-		outStr := labelsToString(labels[labelsOffset:])
-		if inStr == outStr {
-			logger.Infof("\nRelabel  In: %s\nRelabel Out: KEPT AS IS - no change", inStr)
-		} else {
-			logger.Infof("\nRelabel  In: %s\nRelabel Out: %s", inStr, outStr)
-		}
-		// Drop labels
-		labels = labels[:labelsOffset]
-	}
 	return labels
 }
 
@@ -109,29 +145,14 @@ func removeEmptyLabels(labels []prompbmarshal.Label, labelsOffset int) []prompbm
 	return dst
 }
 
-// RemoveMetaLabels removes all the `__meta_` labels from src and puts the rest of labels to dst.
-//
-// See https://www.robustperception.io/life-of-a-label fo details.
-func RemoveMetaLabels(dst, src []prompbmarshal.Label) []prompbmarshal.Label {
-	for i := range src {
-		label := &src[i]
-		if strings.HasPrefix(label.Name, "__meta_") {
-			continue
-		}
-		dst = append(dst, *label)
-	}
-	return dst
-}
-
 // FinalizeLabels removes labels with "__" in the beginning (except of "__name__").
 func FinalizeLabels(dst, src []prompbmarshal.Label) []prompbmarshal.Label {
-	for i := range src {
-		label := &src[i]
+	for _, label := range src {
 		name := label.Name
 		if strings.HasPrefix(name, "__") && name != "__name__" {
 			continue
 		}
-		dst = append(dst, *label)
+		dst = append(dst, label)
 	}
 	return dst
 }
@@ -141,7 +162,7 @@ func FinalizeLabels(dst, src []prompbmarshal.Label) []prompbmarshal.Label {
 // See https://prometheus.io/docs/prometheus/latest/configuration/configuration/#relabel_config
 func (prc *parsedRelabelConfig) apply(labels []prompbmarshal.Label, labelsOffset int) []prompbmarshal.Label {
 	src := labels[labelsOffset:]
-	if prc.If != nil && !prc.If.Match(src) {
+	if !prc.If.Match(src) {
 		if prc.Action == "keep" {
 			// Drop the target on `if` mismatch for `action: keep`
 			return labels[:labelsOffset]
@@ -151,7 +172,7 @@ func (prc *parsedRelabelConfig) apply(labels []prompbmarshal.Label, labelsOffset
 	}
 	switch prc.Action {
 	case "graphite":
-		metricName := GetLabelValueByName(src, "__name__")
+		metricName := getLabelValue(src, "__name__")
 		gm := graphiteMatchesPool.Get().(*graphiteMatches)
 		var ok bool
 		gm.a, ok = prc.graphiteMatchTemplate.Match(gm.a[:0], metricName)
@@ -164,7 +185,7 @@ func (prc *parsedRelabelConfig) apply(labels []prompbmarshal.Label, labelsOffset
 		bb := relabelBufPool.Get()
 		for _, gl := range prc.graphiteLabelRules {
 			bb.B = gl.grt.Expand(bb.B[:0], gm.a)
-			valueStr := string(bb.B)
+			valueStr := bytesutil.InternBytes(bb.B)
 			labels = setLabelValue(labels, labelsOffset, gl.targetLabel, valueStr)
 		}
 		relabelBufPool.Put(bb)
@@ -172,14 +193,20 @@ func (prc *parsedRelabelConfig) apply(labels []prompbmarshal.Label, labelsOffset
 		return labels
 	case "replace":
 		// Store `replacement` at `target_label` if the `regex` matches `source_labels` joined with `separator`
+		replacement := prc.Replacement
 		bb := relabelBufPool.Get()
+		if prc.hasLabelReferenceInReplacement {
+			// Fill {{labelName}} references in the replacement
+			bb.B = fillLabelReferences(bb.B[:0], replacement, labels[labelsOffset:])
+			replacement = bytesutil.InternBytes(bb.B)
+		}
 		bb.B = concatLabelValues(bb.B[:0], src, prc.SourceLabels, prc.Separator)
-		if prc.Regex == defaultRegexForRelabelConfig && !prc.hasCaptureGroupInTargetLabel {
-			if prc.Replacement == "$1" {
+		if prc.RegexAnchored == defaultRegexForRelabelConfig && !prc.hasCaptureGroupInTargetLabel {
+			if replacement == "$1" {
 				// Fast path for the rule that copies source label values to destination:
 				// - source_labels: [...]
 				//   target_label: foobar
-				valueStr := string(bb.B)
+				valueStr := bytesutil.InternBytes(bb.B)
 				relabelBufPool.Put(bb)
 				return setLabelValue(labels, labelsOffset, prc.TargetLabel, valueStr)
 			}
@@ -188,22 +215,33 @@ func (prc *parsedRelabelConfig) apply(labels []prompbmarshal.Label, labelsOffset
 				// - target_label: foobar
 				//   replacement: something-here
 				relabelBufPool.Put(bb)
-				labels = setLabelValue(labels, labelsOffset, prc.TargetLabel, prc.Replacement)
+				labels = setLabelValue(labels, labelsOffset, prc.TargetLabel, replacement)
 				return labels
 			}
 		}
-		match := prc.Regex.FindSubmatchIndex(bb.B)
-		if match == nil {
-			// Fast path - nothing to replace.
+		sourceStr := bytesutil.ToUnsafeString(bb.B)
+		if !prc.regex.MatchString(sourceStr) {
+			// Fast path - regexp mismatch.
 			relabelBufPool.Put(bb)
 			return labels
 		}
-		sourceStr := bytesutil.ToUnsafeString(bb.B)
+		var valueStr string
+		if replacement == prc.Replacement {
+			// Fast path - the replacement wasn't modified, so it is safe calling stringReplacer.Transform.
+			valueStr = prc.stringReplacer.Transform(sourceStr)
+		} else {
+			// Slow path - the replacement has been modified, so the valueStr must be calculated
+			// from scratch based on the new replacement value.
+			match := prc.RegexAnchored.FindSubmatchIndex(bb.B)
+			valueStr = prc.expandCaptureGroups(replacement, sourceStr, match)
+		}
 		nameStr := prc.TargetLabel
 		if prc.hasCaptureGroupInTargetLabel {
+			// Slow path - target_label contains regex capture groups, so the target_label
+			// must be calculated from the regex match.
+			match := prc.RegexAnchored.FindSubmatchIndex(bb.B)
 			nameStr = prc.expandCaptureGroups(nameStr, sourceStr, match)
 		}
-		valueStr := prc.expandCaptureGroups(prc.Replacement, sourceStr, match)
 		relabelBufPool.Put(bb)
 		return setLabelValue(labels, labelsOffset, nameStr, valueStr)
 	case "replace_all":
@@ -211,13 +249,39 @@ func (prc *parsedRelabelConfig) apply(labels []prompbmarshal.Label, labelsOffset
 		// and store the result at `target_label`
 		bb := relabelBufPool.Get()
 		bb.B = concatLabelValues(bb.B[:0], src, prc.SourceLabels, prc.Separator)
-		sourceStr := string(bb.B)
+		sourceStr := bytesutil.InternBytes(bb.B)
 		relabelBufPool.Put(bb)
-		valueStr, ok := prc.replaceStringSubmatches(sourceStr, prc.Replacement, prc.hasCaptureGroupInReplacement)
-		if ok {
+		valueStr := prc.replaceStringSubmatchesFast(sourceStr)
+		if valueStr != sourceStr {
 			labels = setLabelValue(labels, labelsOffset, prc.TargetLabel, valueStr)
 		}
 		return labels
+	case "keep_if_contains":
+		// Keep the entry if target_label contains all the label values listed in source_labels.
+		// For example, the following relabeling rule would leave the entry if __meta_consul_tags
+		// contains values of __meta_required_tag1 and __meta_required_tag2:
+		//
+		//   - action: keep_if_contains
+		//     target_label: __meta_consul_tags
+		//     source_labels: [__meta_required_tag1, __meta_required_tag2]
+		//
+		if containsAllLabelValues(src, prc.TargetLabel, prc.SourceLabels) {
+			return labels
+		}
+		return labels[:labelsOffset]
+	case "drop_if_contains":
+		// Drop the entry if target_label contains all the label values listed in source_labels.
+		// For example, the following relabeling rule would drop the entry if __meta_consul_tags
+		// contains values of __meta_required_tag1 and __meta_required_tag2:
+		//
+		//   - action: drop_if_contains
+		//     target_label: __meta_consul_tags
+		//     source_labels: [__meta_required_tag1, __meta_required_tag2]
+		//
+		if containsAllLabelValues(src, prc.TargetLabel, prc.SourceLabels) {
+			return labels[:labelsOffset]
+		}
+		return labels
 	case "keep_if_equal":
 		// Keep the entry if all the label values in source_labels are equal.
 		// For example:
@@ -242,9 +306,31 @@ func (prc *parsedRelabelConfig) apply(labels []prompbmarshal.Label, labelsOffset
 			return labels[:labelsOffset]
 		}
 		return labels
+	case "keepequal":
+		// Keep the entry if `source_labels` joined with `separator` matches `target_label`
+		bb := relabelBufPool.Get()
+		bb.B = concatLabelValues(bb.B[:0], src, prc.SourceLabels, prc.Separator)
+		targetValue := getLabelValue(labels[labelsOffset:], prc.TargetLabel)
+		keep := string(bb.B) == targetValue
+		relabelBufPool.Put(bb)
+		if keep {
+			return labels
+		}
+		return labels[:labelsOffset]
+	case "dropequal":
+		// Drop the entry if `source_labels` joined with `separator` doesn't match `target_label`
+		bb := relabelBufPool.Get()
+		bb.B = concatLabelValues(bb.B[:0], src, prc.SourceLabels, prc.Separator)
+		targetValue := getLabelValue(labels[labelsOffset:], prc.TargetLabel)
+		drop := string(bb.B) == targetValue
+		relabelBufPool.Put(bb)
+		if !drop {
+			return labels
+		}
+		return labels[:labelsOffset]
 	case "keep":
 		// Keep the target if `source_labels` joined with `separator` match the `regex`.
-		if prc.Regex == defaultRegexForRelabelConfig {
+		if prc.RegexAnchored == defaultRegexForRelabelConfig {
 			// Fast path for the case with `if` and without explicitly set `regex`:
 			//
 			// - action: keep
@@ -254,7 +340,7 @@ func (prc *parsedRelabelConfig) apply(labels []prompbmarshal.Label, labelsOffset
 		}
 		bb := relabelBufPool.Get()
 		bb.B = concatLabelValues(bb.B[:0], src, prc.SourceLabels, prc.Separator)
-		keep := prc.matchString(bytesutil.ToUnsafeString(bb.B))
+		keep := prc.regex.MatchString(bytesutil.ToUnsafeString(bb.B))
 		relabelBufPool.Put(bb)
 		if !keep {
 			return labels[:labelsOffset]
@@ -262,7 +348,7 @@ func (prc *parsedRelabelConfig) apply(labels []prompbmarshal.Label, labelsOffset
 		return labels
 	case "drop":
 		// Drop the target if `source_labels` joined with `separator` don't match the `regex`.
-		if prc.Regex == defaultRegexForRelabelConfig {
+		if prc.RegexAnchored == defaultRegexForRelabelConfig {
 			// Fast path for the case with `if` and without explicitly set `regex`:
 			//
 			// - action: drop
@@ -272,7 +358,7 @@ func (prc *parsedRelabelConfig) apply(labels []prompbmarshal.Label, labelsOffset
 		}
 		bb := relabelBufPool.Get()
 		bb.B = concatLabelValues(bb.B[:0], src, prc.SourceLabels, prc.Separator)
-		drop := prc.matchString(bytesutil.ToUnsafeString(bb.B))
+		drop := prc.regex.MatchString(bytesutil.ToUnsafeString(bb.B))
 		relabelBufPool.Put(bb)
 		if drop {
 			return labels[:labelsOffset]
@@ -288,45 +374,44 @@ func (prc *parsedRelabelConfig) apply(labels []prompbmarshal.Label, labelsOffset
 		return setLabelValue(labels, labelsOffset, prc.TargetLabel, value)
 	case "labelmap":
 		// Replace label names with the `replacement` if they match `regex`
-		for i := range src {
-			label := &src[i]
-			labelName, ok := prc.replaceFullString(label.Name, prc.Replacement, prc.hasCaptureGroupInReplacement)
-			if ok {
+		for _, label := range src {
+			labelName := prc.replaceFullStringFast(label.Name)
+			if labelName != label.Name {
 				labels = setLabelValue(labels, labelsOffset, labelName, label.Value)
 			}
 		}
 		return labels
 	case "labelmap_all":
-		// Replace all the occurences of `regex` at label names with `replacement`
+		// Replace all the occurrences of `regex` at label names with `replacement`
 		for i := range src {
 			label := &src[i]
-			label.Name, _ = prc.replaceStringSubmatches(label.Name, prc.Replacement, prc.hasCaptureGroupInReplacement)
+			label.Name = prc.replaceStringSubmatchesFast(label.Name)
 		}
 		return labels
 	case "labeldrop":
 		// Drop labels with names matching the `regex`
 		dst := labels[:labelsOffset]
-		for i := range src {
-			label := &src[i]
-			if !prc.matchString(label.Name) {
-				dst = append(dst, *label)
+		re := prc.regex
+		for _, label := range src {
+			if !re.MatchString(label.Name) {
+				dst = append(dst, label)
 			}
 		}
 		return dst
 	case "labelkeep":
 		// Keep labels with names matching the `regex`
 		dst := labels[:labelsOffset]
-		for i := range src {
-			label := &src[i]
-			if prc.matchString(label.Name) {
-				dst = append(dst, *label)
+		re := prc.regex
+		for _, label := range src {
+			if re.MatchString(label.Name) {
+				dst = append(dst, label)
 			}
 		}
 		return dst
 	case "uppercase":
 		bb := relabelBufPool.Get()
 		bb.B = concatLabelValues(bb.B[:0], src, prc.SourceLabels, prc.Separator)
-		valueStr := string(bb.B)
+		valueStr := bytesutil.InternBytes(bb.B)
 		relabelBufPool.Put(bb)
 		valueStr = strings.ToUpper(valueStr)
 		labels = setLabelValue(labels, labelsOffset, prc.TargetLabel, valueStr)
@@ -334,7 +419,7 @@ func (prc *parsedRelabelConfig) apply(labels []prompbmarshal.Label, labelsOffset
 	case "lowercase":
 		bb := relabelBufPool.Get()
 		bb.B = concatLabelValues(bb.B[:0], src, prc.SourceLabels, prc.Separator)
-		valueStr := string(bb.B)
+		valueStr := bytesutil.InternBytes(bb.B)
 		relabelBufPool.Put(bb)
 		valueStr = strings.ToLower(valueStr)
 		labels = setLabelValue(labels, labelsOffset, prc.TargetLabel, valueStr)
@@ -345,16 +430,23 @@ func (prc *parsedRelabelConfig) apply(labels []prompbmarshal.Label, labelsOffset
 	}
 }
 
-func (prc *parsedRelabelConfig) replaceFullString(s, replacement string, hasCaptureGroupInReplacement bool) (string, bool) {
+// replaceFullStringFast replaces s with the replacement if s matches '^regex$'.
+//
+// s is returned as is if it doesn't match '^regex$'.
+func (prc *parsedRelabelConfig) replaceFullStringFast(s string) string {
 	prefix, complete := prc.regexOriginal.LiteralPrefix()
-	if complete && !hasCaptureGroupInReplacement {
+	replacement := prc.Replacement
+	if complete && !prc.hasCaptureGroupInReplacement {
 		if s == prefix {
-			return replacement, true
+			// Fast path - s matches literal regex
+			return replacement
 		}
-		return s, false
+		// Fast path - s doesn't match literal regex
+		return s
 	}
 	if !strings.HasPrefix(s, prefix) {
-		return s, false
+		// Fast path - s doesn't match literal prefix from regex
+		return s
 	}
 	if replacement == "$1" {
 		// Fast path for commonly used rule for deleting label prefixes such as:
@@ -368,82 +460,80 @@ func (prc *parsedRelabelConfig) replaceFullString(s, replacement string, hasCapt
 			reSuffix := reStr[len(prefix):]
 			switch reSuffix {
 			case "(.*)":
-				return suffix, true
+				return suffix
 			case "(.+)":
 				if len(suffix) > 0 {
-					return suffix, true
+					return suffix
 				}
-				return s, false
+				return s
 			}
 		}
 	}
+	if !prc.regex.MatchString(s) {
+		// Fast path - regex mismatch
+		return s
+	}
+	// Slow path - handle the rest of cases.
+	return prc.stringReplacer.Transform(s)
+}
+
+// replaceFullStringSlow replaces s with the replacement if s matches '^regex$'.
+//
+// s is returned as is if it doesn't match '^regex$'.
+func (prc *parsedRelabelConfig) replaceFullStringSlow(s string) string {
 	// Slow path - regexp processing
-	match := prc.Regex.FindStringSubmatchIndex(s)
+	match := prc.RegexAnchored.FindStringSubmatchIndex(s)
 	if match == nil {
-		return s, false
+		return s
 	}
-	bb := relabelBufPool.Get()
-	bb.B = prc.Regex.ExpandString(bb.B[:0], replacement, s, match)
-	result := string(bb.B)
-	relabelBufPool.Put(bb)
-	return result, true
+	return prc.expandCaptureGroups(prc.Replacement, s, match)
 }
 
-func (prc *parsedRelabelConfig) replaceStringSubmatches(s, replacement string, hasCaptureGroupInReplacement bool) (string, bool) {
-	re := prc.regexOriginal
-	prefix, complete := re.LiteralPrefix()
-	if complete && !hasCaptureGroupInReplacement {
-		if !strings.Contains(s, prefix) {
-			return s, false
-		}
-		return strings.ReplaceAll(s, prefix, replacement), true
-	}
-	if !re.MatchString(s) {
-		return s, false
-	}
-	return re.ReplaceAllString(s, replacement), true
-}
-
-func (prc *parsedRelabelConfig) matchString(s string) bool {
+// replaceStringSubmatchesFast replaces all the regex matches with the replacement in s.
+func (prc *parsedRelabelConfig) replaceStringSubmatchesFast(s string) string {
 	prefix, complete := prc.regexOriginal.LiteralPrefix()
-	if complete {
-		return prefix == s
+	if complete && !prc.hasCaptureGroupInReplacement && !strings.Contains(s, prefix) {
+		// Fast path - zero regex matches in s.
+		return s
 	}
-	if !strings.HasPrefix(s, prefix) {
-		return false
-	}
-	reStr := prc.regexOriginal.String()
-	if strings.HasPrefix(reStr, prefix) {
-		// Fast path for `foo.*` and `bar.+` regexps
-		reSuffix := reStr[len(prefix):]
-		switch reSuffix {
-		case ".+", "(.+)":
-			return len(s) > len(prefix)
-		case ".*", "(.*)":
-			return true
-		}
-	}
-	return prc.Regex.MatchString(s)
+	// Slow path - replace all the regex matches in s with the replacement.
+	return prc.submatchReplacer.Transform(s)
+}
+
+// replaceStringSubmatchesSlow replaces all the regex matches with the replacement in s.
+func (prc *parsedRelabelConfig) replaceStringSubmatchesSlow(s string) string {
+	return prc.regexOriginal.ReplaceAllString(s, prc.Replacement)
 }
 
 func (prc *parsedRelabelConfig) expandCaptureGroups(template, source string, match []int) string {
 	bb := relabelBufPool.Get()
-	bb.B = prc.Regex.ExpandString(bb.B[:0], template, source, match)
-	s := string(bb.B)
+	bb.B = prc.RegexAnchored.ExpandString(bb.B[:0], template, source, match)
+	s := bytesutil.InternBytes(bb.B)
 	relabelBufPool.Put(bb)
 	return s
 }
 
 var relabelBufPool bytesutil.ByteBufferPool
 
+func containsAllLabelValues(labels []prompbmarshal.Label, targetLabel string, sourceLabels []string) bool {
+	targetLabelValue := getLabelValue(labels, targetLabel)
+	for _, sourceLabel := range sourceLabels {
+		v := getLabelValue(labels, sourceLabel)
+		if !strings.Contains(targetLabelValue, v) {
+			return false
+		}
+	}
+	return true
+}
+
 func areEqualLabelValues(labels []prompbmarshal.Label, labelNames []string) bool {
 	if len(labelNames) < 2 {
 		logger.Panicf("BUG: expecting at least 2 labelNames; got %d", len(labelNames))
 		return false
 	}
-	labelValue := GetLabelValueByName(labels, labelNames[0])
+	labelValue := getLabelValue(labels, labelNames[0])
 	for _, labelName := range labelNames[1:] {
-		v := GetLabelValueByName(labels, labelName)
+		v := getLabelValue(labels, labelName)
 		if v != labelValue {
 			return false
 		}
@@ -456,10 +546,8 @@ func concatLabelValues(dst []byte, labels []prompbmarshal.Label, labelNames []st
 		return dst
 	}
 	for _, labelName := range labelNames {
-		label := GetLabelByName(labels, labelName)
-		if label != nil {
-			dst = append(dst, label.Value...)
-		}
+		labelValue := getLabelValue(labels, labelName)
+		dst = append(dst, labelValue...)
 		dst = append(dst, separator...)
 	}
 	return dst[:len(dst)-len(separator)]
@@ -477,6 +565,15 @@ func setLabelValue(labels []prompbmarshal.Label, labelsOffset int, name, value s
 	return labels
 }
 
+func getLabelValue(labels []prompbmarshal.Label, name string) string {
+	for _, label := range labels {
+		if label.Name == name {
+			return label.Value
+		}
+	}
+	return ""
+}
+
 // GetLabelByName returns label with the given name from labels.
 func GetLabelByName(labels []prompbmarshal.Label, name string) *prompbmarshal.Label {
 	for i := range labels {
@@ -488,47 +585,34 @@ func GetLabelByName(labels []prompbmarshal.Label, name string) *prompbmarshal.La
 	return nil
 }
 
-// GetLabelValueByName returns value for label with the given name from labels.
-//
-// It returns empty string for non-existing label.
-func GetLabelValueByName(labels []prompbmarshal.Label, name string) string {
-	label := GetLabelByName(labels, name)
-	if label == nil {
-		return ""
-	}
-	return label.Value
-}
-
 // CleanLabels sets label.Name and label.Value to an empty string for all the labels.
 //
 // This should help GC cleaning up label.Name and label.Value strings.
 func CleanLabels(labels []prompbmarshal.Label) {
-	for i := range labels {
-		label := &labels[i]
-		label.Name = ""
-		label.Value = ""
-	}
+	clear(labels)
 }
 
-func labelsToString(labels []prompbmarshal.Label) string {
+// LabelsToString returns Prometheus string representation for the given labels.
+//
+// Labels in the returned string are sorted by name,
+// while the __name__ label is put in front of {} labels.
+func LabelsToString(labels []prompbmarshal.Label) string {
 	labelsCopy := append([]prompbmarshal.Label{}, labels...)
 	SortLabels(labelsCopy)
 	mname := ""
-	for _, label := range labelsCopy {
+	for i, label := range labelsCopy {
 		if label.Name == "__name__" {
 			mname = label.Value
+			labelsCopy = append(labelsCopy[:i], labelsCopy[i+1:]...)
 			break
 		}
 	}
-	if mname != "" && len(labelsCopy) <= 1 {
+	if mname != "" && len(labelsCopy) == 0 {
 		return mname
 	}
 	b := []byte(mname)
 	b = append(b, '{')
 	for i, label := range labelsCopy {
-		if label.Name == "__name__" {
-			continue
-		}
 		b = append(b, label.Name...)
 		b = append(b, '=')
 		b = strconv.AppendQuote(b, label.Value)
@@ -539,3 +623,70 @@ func labelsToString(labels []prompbmarshal.Label) string {
 	b = append(b, '}')
 	return string(b)
 }
+
+// SortLabels sorts labels in alphabetical order.
+func SortLabels(labels []prompbmarshal.Label) {
+	x := promutils.GetLabels()
+	labelsOrig := x.Labels
+	x.Labels = labels
+	x.Sort()
+	x.Labels = labelsOrig
+	promutils.PutLabels(x)
+}
+
+func fillLabelReferences(dst []byte, replacement string, labels []prompbmarshal.Label) []byte {
+	s := replacement
+	for len(s) > 0 {
+		n := strings.Index(s, "{{")
+		if n < 0 {
+			return append(dst, s...)
+		}
+		dst = append(dst, s[:n]...)
+		s = s[n+2:]
+		n = strings.Index(s, "}}")
+		if n < 0 {
+			dst = append(dst, "{{"...)
+			return append(dst, s...)
+		}
+		labelName := s[:n]
+		s = s[n+2:]
+		labelValue := getLabelValue(labels, labelName)
+		dst = append(dst, labelValue...)
+	}
+	return dst
+}
+
+// SanitizeLabelName replaces unsupported by Prometheus chars in label names with _.
+//
+// See https://prometheus.io/docs/concepts/data_model/#metric-names-and-labels
+func SanitizeLabelName(name string) string {
+	return labelNameSanitizer.Transform(name)
+}
+
+// SplitMetricNameToTokens returns tokens generated from metric name divided by unsupported Prometheus characters
+//
+// See https://prometheus.io/docs/concepts/data_model/#metric-names-and-labels
+func SplitMetricNameToTokens(name string) []string {
+	return nonAlphaNumChars.Split(name, -1)
+}
+
+var nonAlphaNumChars = regexp.MustCompile(`[^a-zA-Z0-9]`)
+
+var labelNameSanitizer = bytesutil.NewFastStringTransformer(func(s string) string {
+	return unsupportedLabelNameChars.ReplaceAllLiteralString(s, "_")
+})
+
+var unsupportedLabelNameChars = regexp.MustCompile(`[^a-zA-Z0-9_]`)
+
+// SanitizeMetricName replaces unsupported by Prometheus chars in metric names with _.
+//
+// See https://prometheus.io/docs/concepts/data_model/#metric-names-and-labels
+func SanitizeMetricName(value string) string {
+	return metricNameSanitizer.Transform(value)
+}
+
+var metricNameSanitizer = bytesutil.NewFastStringTransformer(func(s string) string {
+	return unsupportedMetricNameChars.ReplaceAllLiteralString(s, "_")
+})
+
+var unsupportedMetricNameChars = regexp.MustCompile(`[^a-zA-Z0-9_:]`)
