@@ -3,11 +3,11 @@ package azure
 import (
 	"encoding/json"
 	"fmt"
+	"net/http"
 	"net/url"
 	"sync"
 
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/cgroup"
-	"github.com/VictoriaMetrics/fasthttp"
 )
 
 // virtualMachine represents an Azure virtual machine (which can also be created by a VMSS)
@@ -29,9 +29,14 @@ type vmIPAddress struct {
 }
 
 type virtualMachineProperties struct {
-	NetworkProfile networkProfile `json:"networkProfile,omitempty"`
-	OsProfile      osProfile      `json:"osProfile,omitempty"`
-	StorageProfile storageProfile `json:"storageProfile,omitempty"`
+	NetworkProfile  networkProfile  `json:"networkProfile,omitempty"`
+	OsProfile       osProfile       `json:"osProfile,omitempty"`
+	StorageProfile  storageProfile  `json:"storageProfile,omitempty"`
+	HardwareProfile hardwareProfile `json:"hardwareProfile,omitempty"`
+}
+
+type hardwareProfile struct {
+	VMSize string `json:"vmSize,omitempty"`
 }
 
 type storageProfile struct {
@@ -45,6 +50,7 @@ type osDisk struct {
 type osProfile struct {
 	ComputerName string `json:"computerName,omitempty"`
 }
+
 type networkProfile struct {
 	// NetworkInterfaces - Specifies the list of resource Ids for the network interfaces associated with the virtual machine.
 	NetworkInterfaces []networkInterfaceReference `json:"networkInterfaces,omitempty"`
@@ -64,7 +70,7 @@ type listAPIResponse struct {
 func visitAllAPIObjects(ac *apiConfig, apiURL string, cb func(data json.RawMessage) error) error {
 	nextLinkURI := apiURL
 	for {
-		resp, err := ac.c.GetAPIResponseWithReqParams(nextLinkURI, func(request *fasthttp.Request) {
+		resp, err := ac.c.GetAPIResponseWithReqParams(nextLinkURI, func(request *http.Request) {
 			request.Header.Set("Authorization", "Bearer "+ac.mustGetAuthToken())
 		})
 		if err != nil {
@@ -90,8 +96,10 @@ func visitAllAPIObjects(ac *apiConfig, apiURL string, cb func(data json.RawMessa
 			return fmt.Errorf("cannot parse nextLink from response %q: %w", lar.NextLink, err)
 		}
 
-		if nextURL.Host != "" && nextURL.Host != ac.c.APIServer() {
-			return fmt.Errorf("unexpected nextLink host %q, expecting %q", nextURL.Host, ac.c.APIServer())
+		// Sometimes Azure will respond a host with a port. Since all possible apiServer defined in cloudEnvironments do not include a port,
+		// it is best to check the host without the port. See https://github.com/VictoriaMetrics/VictoriaMetrics/issues/6912
+		if nextURL.Host != "" && nextURL.Hostname() != ac.apiServerHost {
+			return fmt.Errorf("unexpected nextLink host %q, expecting %q", nextURL.Hostname(), ac.apiServerHost)
 		}
 
 		nextLinkURI = nextURL.RequestURI()
@@ -182,9 +190,16 @@ type scaleSet struct {
 }
 
 // See https://docs.microsoft.com/en-us/rest/api/compute/virtual-machine-scale-sets/list-all
+// and https://learn.microsoft.com/en-us/rest/api/compute/virtual-machine-scale-sets/list (need resourceGroup)
 func listScaleSetRefs(ac *apiConfig) ([]scaleSet, error) {
 	// https://management.azure.com/subscriptions/{subscriptionId}/providers/Microsoft.Compute/virtualMachineScaleSets?api-version=2022-03-01
-	apiURL := "/subscriptions/" + ac.subscriptionID + "/providers/Microsoft.Compute/virtualMachineScaleSets?api-version=2022-03-01"
+	apiURL := "/subscriptions/" + ac.subscriptionID
+	if ac.resourceGroup != "" {
+		// special case filter by resourceGroup
+		// https://management.azure.com/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets?api-version=2022-03-01
+		apiURL += "/resourceGroups/" + ac.resourceGroup
+	}
+	apiURL += "/providers/Microsoft.Compute/virtualMachineScaleSets?api-version=2022-03-01"
 	var sss []scaleSet
 	err := visitAllAPIObjects(ac, apiURL, func(data json.RawMessage) error {
 		var ss scaleSet
