@@ -22,5 +22,11 @@ func TestGetFiltersQueryArg(t *testing.T) {
 			Name:   "xxx",
 			Values: []string{"aa"},
 		},
-	}, "%7B%22name%22%3A%7B%22bar%22%3Atrue%2C%22foo%22%3Atrue%7D%2C%22xxx%22%3A%7B%22aa%22%3Atrue%7D%7D")
+	}, "%7B%22name%22%3A%5B%22foo%22%2C%22bar%22%5D%2C%22xxx%22%3A%5B%22aa%22%5D%7D")
+	f([]Filter{
+		{
+			Name:   "desired-state",
+			Values: []string{"running", "shutdown"},
+		},
+	}, "%7B%22desired-state%22%3A%5B%22running%22%2C%22shutdown%22%5D%7D")
 }
