@@ -4,8 +4,7 @@ import (
 	"bytes"
 	"testing"
 
-	"github.com/VictoriaMetrics/VictoriaMetrics/lib/prompbmarshal"
-	"github.com/VictoriaMetrics/VictoriaMetrics/lib/promscrape/discoveryutils"
+	"github.com/VictoriaMetrics/VictoriaMetrics/lib/promutils"
 )
 
 func TestParseEndpointSliceListFail(t *testing.T) {
@@ -13,10 +12,10 @@ func TestParseEndpointSliceListFail(t *testing.T) {
 		r := bytes.NewBufferString(data)
 		objectsByKey, _, err := parseEndpointSliceList(r)
 		if err == nil {
-			t.Errorf("unexpected result, test must fail! data: %s", data)
+			t.Fatalf("unexpected result, test must fail! data: %s", data)
 		}
 		if len(objectsByKey) != 0 {
-			t.Errorf("EndpointSliceList must be emptry, got: %v", objectsByKey)
+			t.Fatalf("EndpointSliceList must be emptry, got: %v", objectsByKey)
 		}
 	}
 
@@ -157,16 +156,15 @@ func TestParseEndpointSliceListSuccess(t *testing.T) {
 	r := bytes.NewBufferString(data)
 	objectsByKey, meta, err := parseEndpointSliceList(r)
 	if err != nil {
-		t.Errorf("cannot parse data for EndpointSliceList: %v", err)
-		return
+		t.Fatalf("cannot parse data for EndpointSliceList: %v", err)
 	}
 	expectedResourceVersion := "1177"
 	if meta.ResourceVersion != expectedResourceVersion {
 		t.Fatalf("unexpected resource version; got %s; want %s", meta.ResourceVersion, expectedResourceVersion)
 	}
 	sortedLabelss := getSortedLabelss(objectsByKey)
-	expectedLabelss := [][]prompbmarshal.Label{
-		discoveryutils.GetSortedLabels(map[string]string{
+	expectedLabelss := []*promutils.Labels{
+		promutils.NewLabelsFromMap(map[string]string{
 			"__address__": "10.244.0.3:53",
 			"__meta_kubernetes_endpointslice_address_target_kind":                                                "Pod",
 			"__meta_kubernetes_endpointslice_address_target_name":                                                "coredns-66bff467f8-z8czk",
@@ -186,7 +184,7 @@ func TestParseEndpointSliceListSuccess(t *testing.T) {
 			"__meta_kubernetes_endpointslice_port_protocol":                                                      "UDP",
 			"__meta_kubernetes_namespace":                                                                        "kube-system",
 		}),
-		discoveryutils.GetSortedLabels(map[string]string{
+		promutils.NewLabelsFromMap(map[string]string{
 			"__address__": "10.244.0.3:9153",
 			"__meta_kubernetes_endpointslice_address_target_kind":                                                "Pod",
 			"__meta_kubernetes_endpointslice_address_target_name":                                                "coredns-66bff467f8-z8czk",
@@ -206,7 +204,7 @@ func TestParseEndpointSliceListSuccess(t *testing.T) {
 			"__meta_kubernetes_endpointslice_port_protocol":                                                      "TCP",
 			"__meta_kubernetes_namespace":                                                                        "kube-system",
 		}),
-		discoveryutils.GetSortedLabels(map[string]string{
+		promutils.NewLabelsFromMap(map[string]string{
 			"__address__": "172.18.0.2:6443",
 			"__meta_kubernetes_endpointslice_address_type":                            "IPv4",
 			"__meta_kubernetes_endpointslice_endpoint_conditions_ready":               "true",
@@ -227,16 +225,17 @@ func TestParseEndpointSliceListSuccess(t *testing.T) {
 
 func TestGetEndpointsliceLabels(t *testing.T) {
 	type testArgs struct {
-		containerPorts map[string][]ContainerPort
-		endpointPorts  []EndpointPort
+		initContainerPorts map[string][]ContainerPort
+		containerPorts     map[string][]ContainerPort
+		endpointPorts      []EndpointPort
 	}
-	f := func(t *testing.T, args testArgs, wantLabels [][]prompbmarshal.Label) {
+	f := func(t *testing.T, args testArgs, wantLabels []*promutils.Labels) {
 		t.Helper()
 		eps := EndpointSlice{
 			Metadata: ObjectMeta{
 				Name:      "test-eps",
 				Namespace: "default",
-				Labels: discoveryutils.GetSortedLabels(map[string]string{
+				Labels: promutils.NewLabelsFromMap(map[string]string{
 					"kubernetes.io/service-name": "test-svc",
 				}),
 			},
@@ -295,16 +294,23 @@ func TestGetEndpointsliceLabels(t *testing.T) {
 		}
 		node := Node{
 			Metadata: ObjectMeta{
-				Labels: []prompbmarshal.Label{
-					{
-						Name:  "node-label",
-						Value: "xyz",
-					},
-				},
+				Labels: promutils.NewLabelsFromMap(map[string]string{"node-label": "xyz"}),
 			},
 		}
+		for cn, ports := range args.initContainerPorts {
+			pod.Spec.InitContainers = append(pod.Spec.InitContainers, Container{
+				Name:          cn,
+				Image:         "test-init-image",
+				Ports:         ports,
+				RestartPolicy: "Always",
+			})
+		}
 		for cn, ports := range args.containerPorts {
-			pod.Spec.Containers = append(pod.Spec.Containers, Container{Name: cn, Ports: ports})
+			pod.Spec.Containers = append(pod.Spec.Containers, Container{
+				Name:  cn,
+				Image: "test-image",
+				Ports: ports,
+			})
 		}
 		var gw groupWatcher
 		gw.m = map[string]*urlWatcher{
@@ -328,10 +334,11 @@ func TestGetEndpointsliceLabels(t *testing.T) {
 			},
 		}
 		gw.attachNodeMetadata = true
-		var sortedLabelss [][]prompbmarshal.Label
+		var sortedLabelss []*promutils.Labels
 		gotLabels := eps.getTargetLabels(&gw)
 		for _, lbs := range gotLabels {
-			sortedLabelss = append(sortedLabelss, discoveryutils.GetSortedLabels(lbs))
+			lbs.Sort()
+			sortedLabelss = append(sortedLabelss, lbs)
 		}
 		if !areEqualLabelss(sortedLabelss, wantLabels) {
 			t.Fatalf("unexpected labels:\ngot\n%v\nwant\n%v", sortedLabelss, wantLabels)
@@ -347,8 +354,8 @@ func TestGetEndpointsliceLabels(t *testing.T) {
 					Protocol: "foobar",
 				},
 			},
-		}, [][]prompbmarshal.Label{
-			discoveryutils.GetSortedLabels(map[string]string{
+		}, []*promutils.Labels{
+			promutils.NewLabelsFromMap(map[string]string{
 				"__address__": "10.13.15.15:8081",
 				"__meta_kubernetes_endpointslice_address_target_kind":                     "Pod",
 				"__meta_kubernetes_endpointslice_address_target_name":                     "test-pod",
@@ -395,8 +402,8 @@ func TestGetEndpointsliceLabels(t *testing.T) {
 					Protocol: "https",
 				},
 			},
-		}, [][]prompbmarshal.Label{
-			discoveryutils.GetSortedLabels(map[string]string{
+		}, []*promutils.Labels{
+			promutils.NewLabelsFromMap(map[string]string{
 				"__address__": "10.13.15.15:8081",
 				"__meta_kubernetes_endpointslice_address_target_kind":                     "Pod",
 				"__meta_kubernetes_endpointslice_address_target_name":                     "test-pod",
@@ -426,26 +433,34 @@ func TestGetEndpointsliceLabels(t *testing.T) {
 				"__meta_kubernetes_service_name":                                          "test-svc",
 				"__meta_kubernetes_service_type":                                          "service-type",
 			}),
-			discoveryutils.GetSortedLabels(map[string]string{
-				"__address__":                                    "192.168.15.1:8428",
-				"__meta_kubernetes_namespace":                    "default",
-				"__meta_kubernetes_node_label_node_label":        "xyz",
-				"__meta_kubernetes_node_labelpresent_node_label": "true",
-				"__meta_kubernetes_node_name":                    "test-node",
-				"__meta_kubernetes_pod_container_name":           "metrics",
-				"__meta_kubernetes_pod_container_port_name":      "http-metrics",
-				"__meta_kubernetes_pod_container_port_number":    "8428",
-				"__meta_kubernetes_pod_container_port_protocol":  "foobar",
-				"__meta_kubernetes_pod_host_ip":                  "4.5.6.7",
-				"__meta_kubernetes_pod_ip":                       "192.168.15.1",
-				"__meta_kubernetes_pod_name":                     "test-pod",
-				"__meta_kubernetes_pod_node_name":                "test-node",
-				"__meta_kubernetes_pod_phase":                    "abc",
-				"__meta_kubernetes_pod_ready":                    "unknown",
-				"__meta_kubernetes_pod_uid":                      "pod-uid",
-				"__meta_kubernetes_service_cluster_ip":           "1.2.3.4",
-				"__meta_kubernetes_service_name":                 "test-svc",
-				"__meta_kubernetes_service_type":                 "service-type",
+			promutils.NewLabelsFromMap(map[string]string{
+				"__address__": "192.168.15.1:8428",
+				"__meta_kubernetes_endpointslice_address_target_kind":                     "Pod",
+				"__meta_kubernetes_endpointslice_address_target_name":                     "test-pod",
+				"__meta_kubernetes_endpointslice_address_type":                            "foobar",
+				"__meta_kubernetes_endpointslice_label_kubernetes_io_service_name":        "test-svc",
+				"__meta_kubernetes_endpointslice_labelpresent_kubernetes_io_service_name": "true",
+				"__meta_kubernetes_endpointslice_name":                                    "test-eps",
+				"__meta_kubernetes_namespace":                                             "default",
+				"__meta_kubernetes_node_label_node_label":                                 "xyz",
+				"__meta_kubernetes_node_labelpresent_node_label":                          "true",
+				"__meta_kubernetes_node_name":                                             "test-node",
+				"__meta_kubernetes_pod_container_image":                                   "test-image",
+				"__meta_kubernetes_pod_container_init":                                    "false",
+				"__meta_kubernetes_pod_container_name":                                    "metrics",
+				"__meta_kubernetes_pod_container_port_name":                               "http-metrics",
+				"__meta_kubernetes_pod_container_port_number":                             "8428",
+				"__meta_kubernetes_pod_container_port_protocol":                           "foobar",
+				"__meta_kubernetes_pod_host_ip":                                           "4.5.6.7",
+				"__meta_kubernetes_pod_ip":                                                "192.168.15.1",
+				"__meta_kubernetes_pod_name":                                              "test-pod",
+				"__meta_kubernetes_pod_node_name":                                         "test-node",
+				"__meta_kubernetes_pod_phase":                                             "abc",
+				"__meta_kubernetes_pod_ready":                                             "unknown",
+				"__meta_kubernetes_pod_uid":                                               "pod-uid",
+				"__meta_kubernetes_service_cluster_ip":                                    "1.2.3.4",
+				"__meta_kubernetes_service_name":                                          "test-svc",
+				"__meta_kubernetes_service_type":                                          "service-type",
 			}),
 		})
 	})
@@ -464,8 +479,8 @@ func TestGetEndpointsliceLabels(t *testing.T) {
 					Protocol: "xabc",
 				},
 			},
-		}, [][]prompbmarshal.Label{
-			discoveryutils.GetSortedLabels(map[string]string{
+		}, []*promutils.Labels{
+			promutils.NewLabelsFromMap(map[string]string{
 				"__address__": "10.13.15.15:8428",
 				"__meta_kubernetes_endpointslice_address_target_kind":                     "Pod",
 				"__meta_kubernetes_endpointslice_address_target_name":                     "test-pod",
@@ -484,6 +499,62 @@ func TestGetEndpointsliceLabels(t *testing.T) {
 				"__meta_kubernetes_node_label_node_label":                                 "xyz",
 				"__meta_kubernetes_node_labelpresent_node_label":                          "true",
 				"__meta_kubernetes_node_name":                                             "test-node",
+				"__meta_kubernetes_pod_container_image":                                   "test-image",
+				"__meta_kubernetes_pod_container_init":                                    "false",
+				"__meta_kubernetes_pod_container_name":                                    "metrics",
+				"__meta_kubernetes_pod_container_port_name":                               "web",
+				"__meta_kubernetes_pod_container_port_number":                             "8428",
+				"__meta_kubernetes_pod_container_port_protocol":                           "sdc",
+				"__meta_kubernetes_pod_host_ip":                                           "4.5.6.7",
+				"__meta_kubernetes_pod_ip":                                                "192.168.15.1",
+				"__meta_kubernetes_pod_name":                                              "test-pod",
+				"__meta_kubernetes_pod_node_name":                                         "test-node",
+				"__meta_kubernetes_pod_phase":                                             "abc",
+				"__meta_kubernetes_pod_ready":                                             "unknown",
+				"__meta_kubernetes_pod_uid":                                               "pod-uid",
+				"__meta_kubernetes_service_cluster_ip":                                    "1.2.3.4",
+				"__meta_kubernetes_service_name":                                          "test-svc",
+				"__meta_kubernetes_service_type":                                          "service-type",
+			}),
+		})
+	})
+
+	t.Run("1 init container port from endpoint", func(t *testing.T) {
+		f(t, testArgs{
+			initContainerPorts: map[string][]ContainerPort{"metrics": {{
+				Name:          "web",
+				ContainerPort: 8428,
+				Protocol:      "sdc",
+			}}},
+			endpointPorts: []EndpointPort{
+				{
+					Name:     "web",
+					Port:     8428,
+					Protocol: "xabc",
+				},
+			},
+		}, []*promutils.Labels{
+			promutils.NewLabelsFromMap(map[string]string{
+				"__address__": "10.13.15.15:8428",
+				"__meta_kubernetes_endpointslice_address_target_kind":                     "Pod",
+				"__meta_kubernetes_endpointslice_address_target_name":                     "test-pod",
+				"__meta_kubernetes_endpointslice_address_type":                            "foobar",
+				"__meta_kubernetes_endpointslice_endpoint_conditions_ready":               "true",
+				"__meta_kubernetes_endpointslice_endpoint_hostname":                       "foo.bar",
+				"__meta_kubernetes_endpointslice_endpoint_topology_present_x":             "true",
+				"__meta_kubernetes_endpointslice_endpoint_topology_x":                     "y",
+				"__meta_kubernetes_endpointslice_label_kubernetes_io_service_name":        "test-svc",
+				"__meta_kubernetes_endpointslice_labelpresent_kubernetes_io_service_name": "true",
+				"__meta_kubernetes_endpointslice_name":                                    "test-eps",
+				"__meta_kubernetes_endpointslice_port":                                    "8428",
+				"__meta_kubernetes_endpointslice_port_name":                               "web",
+				"__meta_kubernetes_endpointslice_port_protocol":                           "xabc",
+				"__meta_kubernetes_namespace":                                             "default",
+				"__meta_kubernetes_node_label_node_label":                                 "xyz",
+				"__meta_kubernetes_node_labelpresent_node_label":                          "true",
+				"__meta_kubernetes_node_name":                                             "test-node",
+				"__meta_kubernetes_pod_container_image":                                   "test-init-image",
+				"__meta_kubernetes_pod_container_init":                                    "true",
 				"__meta_kubernetes_pod_container_name":                                    "metrics",
 				"__meta_kubernetes_pod_container_port_name":                               "web",
 				"__meta_kubernetes_pod_container_port_number":                             "8428",
