@@ -11,7 +11,7 @@ import (
 // It automatically removes old configs which weren't accessed recently.
 type ConfigMap struct {
 	mu sync.Mutex
-	m  map[interface{}]interface{}
+	m  map[any]any
 
 	entriesCount *metrics.Counter
 }
@@ -19,7 +19,7 @@ type ConfigMap struct {
 // NewConfigMap creates ConfigMap
 func NewConfigMap() *ConfigMap {
 	return &ConfigMap{
-		m:            make(map[interface{}]interface{}),
+		m:            make(map[any]any),
 		entriesCount: metrics.GetOrCreateCounter(`vm_promscrape_discoveryutils_configmap_entries_count`),
 	}
 }
@@ -29,7 +29,7 @@ func NewConfigMap() *ConfigMap {
 // Key must be a pointer.
 //
 // It creates new config map with newConfig() call if cm doesn't contain config under the given key.
-func (cm *ConfigMap) Get(key interface{}, newConfig func() (interface{}, error)) (interface{}, error) {
+func (cm *ConfigMap) Get(key any, newConfig func() (any, error)) (any, error) {
 	cm.mu.Lock()
 	defer cm.mu.Unlock()
 
@@ -47,7 +47,7 @@ func (cm *ConfigMap) Get(key interface{}, newConfig func() (interface{}, error))
 }
 
 // Delete deletes config for the given key from cm and returns it.
-func (cm *ConfigMap) Delete(key interface{}) interface{} {
+func (cm *ConfigMap) Delete(key any) any {
 	cm.mu.Lock()
 	defer cm.mu.Unlock()
 
