@@ -2,13 +2,10 @@ package graphite
 
 import (
 	"reflect"
-	"strings"
 	"testing"
-
-	"github.com/VictoriaMetrics/VictoriaMetrics/lib/fasttime"
 )
 
-func TestUnmarshalMetricAndTagsFailure(t *testing.T) {
+func TestUnmarshalMetricAndTags_Failure(t *testing.T) {
 	f := func(s string) {
 		t.Helper()
 		var r Row
@@ -19,16 +16,9 @@ func TestUnmarshalMetricAndTagsFailure(t *testing.T) {
 	}
 	f("")
 	f(";foo=bar")
-	f(" ")
-	f("foo ;bar=baz")
-	f("f oo;bar=baz")
-	f("foo;bar=baz   ")
-	f("foo;bar= baz")
-	f("foo;bar=b az")
-	f("foo;b ar=baz")
 }
 
-func TestUnmarshalMetricAndTagsSuccess(t *testing.T) {
+func TestUnmarshalMetricAndTags_Success(t *testing.T) {
 	f := func(s string, rExpected *Row) {
 		t.Helper()
 		var r Row
@@ -40,10 +30,67 @@ func TestUnmarshalMetricAndTagsSuccess(t *testing.T) {
 			t.Fatalf("unexpected row;\ngot\n%+v\nwant\n%+v", &r, rExpected)
 		}
 	}
+	f(" ", &Row{
+		Metric: " ",
+	})
+	f("foo ;bar=baz", &Row{
+		Metric: "foo ",
+		Tags: []Tag{
+			{
+				Key:   "bar",
+				Value: "baz",
+			},
+		},
+	})
+	f("f oo;bar=baz", &Row{
+		Metric: "f oo",
+		Tags: []Tag{
+			{
+				Key:   "bar",
+				Value: "baz",
+			},
+		},
+	})
+	f("foo;bar=baz   ", &Row{
+		Metric: "foo",
+		Tags: []Tag{
+			{
+				Key:   "bar",
+				Value: "baz   ",
+			},
+		},
+	})
+	f("foo;bar= baz", &Row{
+		Metric: "foo",
+		Tags: []Tag{
+			{
+				Key:   "bar",
+				Value: " baz",
+			},
+		},
+	})
+	f("foo;bar=b az", &Row{
+		Metric: "foo",
+		Tags: []Tag{
+			{
+				Key:   "bar",
+				Value: "b az",
+			},
+		},
+	})
+	f("foo;b ar=baz", &Row{
+		Metric: "foo",
+		Tags: []Tag{
+			{
+				Key:   "b ar",
+				Value: "baz",
+			},
+		},
+	})
 	f("foo", &Row{
 		Metric: "foo",
 	})
-	f("foo;bar=123;baz=aabb", &Row{
+	f("foo;bar=123;baz=aa=bb", &Row{
 		Metric: "foo",
 		Tags: []Tag{
 			{
@@ -52,13 +99,13 @@ func TestUnmarshalMetricAndTagsSuccess(t *testing.T) {
 			},
 			{
 				Key:   "baz",
-				Value: "aabb",
+				Value: "aa=bb",
 			},
 		},
 	})
 }
 
-func TestRowsUnmarshalFailure(t *testing.T) {
+func TestRowsUnmarshal_Failure(t *testing.T) {
 	f := func(s string) {
 		t.Helper()
 		var rows Rows
@@ -74,16 +121,9 @@ func TestRowsUnmarshalFailure(t *testing.T) {
 		}
 	}
 
-	// Missing metric
-	f(" 123 455")
-
 	// Missing value
 	f("aaa")
 
-	// unexpected space in tag value
-	// See https://github.com/VictoriaMetrics/VictoriaMetrics/issues/99
-	f("s;tag1=aaa1;tag2=bb b2;tag3=ccc3 1")
-
 	// invalid value
 	f("aa bb")
 
@@ -91,7 +131,49 @@ func TestRowsUnmarshalFailure(t *testing.T) {
 	f("aa 123 bar")
 }
 
-func TestRowsUnmarshalSuccess(t *testing.T) {
+func TestRowsUnmarshal_SanitizeMetricNamesSuccess(t *testing.T) {
+	f := func(s string, rowsExpected *Rows) {
+		t.Helper()
+
+		sanitizeMetricNameOrig := *sanitizeMetricName
+		*sanitizeMetricName = true
+		defer func() {
+			*sanitizeMetricName = sanitizeMetricNameOrig
+		}()
+
+		var rows Rows
+		rows.Unmarshal(s)
+		if !reflect.DeepEqual(rows.Rows, rowsExpected.Rows) {
+			t.Fatalf("unexpected rows;\ngot\n%+v;\nwant\n%+v", rows.Rows, rowsExpected.Rows)
+		}
+	}
+
+	// Only metric name.
+	f(`foo...b..a.r\a--baz 123`, &Rows{
+		Rows: []Row{{
+			Metric: `foo.b.a.r_a__baz`,
+			Value:  123,
+		}},
+	})
+
+	// Metric name with tags.
+	// Tag values shouldn't be sanitized.
+	f(`s a;ta g..1=a-b..c;tag2 123 456`, &Rows{
+		Rows: []Row{{
+			Metric:    `s_a`,
+			Value:     123,
+			Timestamp: 456,
+			Tags: []Tag{
+				{
+					Key:   "ta_g.1",
+					Value: "a-b..c",
+				},
+			},
+		}},
+	})
+}
+
+func TestRowsUnmarshal_Success(t *testing.T) {
 	f := func(s string, rowsExpected *Rows) {
 		t.Helper()
 		var rows Rows
@@ -103,7 +185,7 @@ func TestRowsUnmarshalSuccess(t *testing.T) {
 		// Try unmarshaling again
 		rows.Unmarshal(s)
 		if !reflect.DeepEqual(rows.Rows, rowsExpected.Rows) {
-			t.Fatalf("unexpected rows;\ngot\n%+v;\nwant\n%+v", rows.Rows, rowsExpected.Rows)
+			t.Fatalf("unexpected rows on second unmarshal;\ngot\n%+v;\nwant\n%+v", rows.Rows, rowsExpected.Rows)
 		}
 
 		rows.Reset()
@@ -119,6 +201,12 @@ func TestRowsUnmarshalSuccess(t *testing.T) {
 	f("\n\r\n", &Rows{})
 
 	// Single line
+	f(" 123 455", &Rows{
+		Rows: []Row{{
+			Metric: "123",
+			Value:  455,
+		}},
+	})
 	f("foobar -123.456 789", &Rows{
 		Rows: []Row{{
 			Metric:    "foobar",
@@ -134,6 +222,26 @@ func TestRowsUnmarshalSuccess(t *testing.T) {
 		}},
 	})
 
+	// Whitespace in metric name, tag name and tag value
+	// See https://github.com/VictoriaMetrics/VictoriaMetrics/issues/3102
+	f("s a;ta g1=aaa1;tag2=bb b2;tag3 1 23", &Rows{
+		Rows: []Row{{
+			Metric:    "s a",
+			Value:     1,
+			Timestamp: 23,
+			Tags: []Tag{
+				{
+					Key:   "ta g1",
+					Value: "aaa1",
+				},
+				{
+					Key:   "tag2",
+					Value: "bb b2",
+				},
+			},
+		}},
+	})
+
 	// Missing timestamp
 	f("aaa 1123", &Rows{
 		Rows: []Row{{
@@ -314,71 +422,3 @@ func TestRowsUnmarshalSuccess(t *testing.T) {
 		}},
 	})
 }
-
-func Test_streamContext_Read(t *testing.T) {
-	f := func(s string, rowsExpected *Rows) {
-		t.Helper()
-		ctx := getStreamContext(strings.NewReader(s))
-		if !ctx.Read() {
-			t.Fatalf("expecting successful read")
-		}
-		uw := getUnmarshalWork()
-		callbackCalls := 0
-		uw.ctx = ctx
-		uw.callback = func(rows []Row) error {
-			callbackCalls++
-			if len(rows) != len(rowsExpected.Rows) {
-				t.Fatalf("different len of expected rows;\ngot\n%+v;\nwant\n%+v", rows, rowsExpected.Rows)
-			}
-			if !reflect.DeepEqual(rows, rowsExpected.Rows) {
-				t.Fatalf("unexpected rows;\ngot\n%+v;\nwant\n%+v", rows, rowsExpected.Rows)
-			}
-			return nil
-		}
-		uw.reqBuf = append(uw.reqBuf[:0], ctx.reqBuf...)
-		ctx.wg.Add(1)
-		uw.Unmarshal()
-		if callbackCalls != 1 {
-			t.Fatalf("unexpected number of callback calls; got %d; want 1", callbackCalls)
-		}
-	}
-
-	// Full line without tags
-	f("aaa 1123 345", &Rows{
-		Rows: []Row{{
-			Metric:    "aaa",
-			Value:     1123,
-			Timestamp: 345 * 1000,
-		}},
-	})
-	// Full line with tags
-	f("aaa;x=y 1123 345", &Rows{
-		Rows: []Row{{
-			Metric: "aaa",
-			Tags: []Tag{{
-				Key:   "x",
-				Value: "y",
-			}},
-			Value:     1123,
-			Timestamp: 345 * 1000,
-		}},
-	})
-	// missing timestamp.
-	// Note that this test may be flaky due to timing issues. TODO: fix it
-	f("aaa 1123", &Rows{
-		Rows: []Row{{
-			Metric:    "aaa",
-			Value:     1123,
-			Timestamp: int64(fasttime.UnixTimestamp()) * 1000,
-		}},
-	})
-	// -1 timestamp. See https://github.com/VictoriaMetrics/VictoriaMetrics/issues/610
-	// Note that this test may be flaky due to timing issues. TODO: fix it.
-	f("aaa 1123 -1", &Rows{
-		Rows: []Row{{
-			Metric:    "aaa",
-			Value:     1123,
-			Timestamp: int64(fasttime.UnixTimestamp()) * 1000,
-		}},
-	})
-}
