@@ -69,7 +69,7 @@ func (ps *partSearch) Init(p *part, tsids []TSID, tr TimeRange) {
 		if isInTest && !sort.SliceIsSorted(tsids, func(i, j int) bool { return tsids[i].Less(&tsids[j]) }) {
 			logger.Panicf("BUG: tsids must be sorted; got %+v", tsids)
 		}
-		// take ownership of of tsids.
+		// take ownership of tsids.
 		ps.tsids = tsids
 	}
 	ps.tr = tr
@@ -120,14 +120,38 @@ func (ps *partSearch) nextTSID() bool {
 	return true
 }
 
+func (ps *partSearch) skipTSIDsSmallerThan(tsid *TSID) bool {
+	if !ps.BlockRef.bh.TSID.Less(tsid) {
+		return true
+	}
+	if !ps.nextTSID() {
+		return false
+	}
+	if !ps.BlockRef.bh.TSID.Less(tsid) {
+		// Fast path: the next TSID isn't smaller than the tsid.
+		return true
+	}
+
+	// Slower path - binary search for the next TSID, which isn't smaller than the tsid.
+	tsids := ps.tsids[ps.tsidIdx:]
+	ps.tsidIdx += sort.Search(len(tsids), func(i int) bool {
+		return !tsids[i].Less(tsid)
+	})
+	if ps.tsidIdx >= len(ps.tsids) {
+		ps.tsidIdx = len(ps.tsids)
+		ps.err = io.EOF
+		return false
+	}
+	ps.BlockRef.bh.TSID = ps.tsids[ps.tsidIdx]
+	ps.tsidIdx++
+	return true
+}
+
 func (ps *partSearch) nextBHS() bool {
 	for len(ps.metaindex) > 0 {
-		// Optimization: skip tsid values smaller than the minimum value
-		// from ps.metaindex.
-		for ps.BlockRef.bh.TSID.Less(&ps.metaindex[0].TSID) {
-			if !ps.nextTSID() {
-				return false
-			}
+		// Optimization: skip tsid values smaller than the minimum value from ps.metaindex.
+		if !ps.skipTSIDsSmallerThan(&ps.metaindex[0].TSID) {
+			return false
 		}
 		// Invariant: ps.BlockRef.bh.TSID >= ps.metaindex[0].TSID
 
@@ -228,24 +252,31 @@ func (ps *partSearch) readIndexBlock(mr *metaindexRow) (*indexBlock, error) {
 }
 
 func (ps *partSearch) searchBHS() bool {
-	for i := range ps.bhs {
-		bh := &ps.bhs[i]
-
-	nextTSID:
-		if bh.TSID.Less(&ps.BlockRef.bh.TSID) {
-			// Skip blocks with small tsid values.
-			continue
+	bhs := ps.bhs
+	for len(bhs) > 0 {
+		// Skip block headers with tsids smaller than the given tsid.
+		tsid := &ps.BlockRef.bh.TSID
+		if bhs[0].TSID.Less(tsid) {
+			n := sort.Search(len(bhs), func(i int) bool {
+				return !bhs[i].TSID.Less(tsid)
+			})
+			if n == len(bhs) {
+				// Nothing found.
+				break
+			}
+			bhs = bhs[n:]
 		}
+		bh := &bhs[0]
 
-		// Invariant: ps.BlockRef.bh.TSID <= bh.TSID
+		// Invariant: tsid <= bh.TSID
 
-		if bh.TSID.MetricID != ps.BlockRef.bh.TSID.MetricID {
-			// ps.BlockRef.bh.TSID < bh.TSID: no more blocks with the given tsid.
+		if bh.TSID.MetricID != tsid.MetricID {
+			// tsid < bh.TSID: no more blocks with the given tsid.
 			// Proceed to the next (bigger) tsid.
-			if !ps.nextTSID() {
+			if !ps.skipTSIDsSmallerThan(&bh.TSID) {
 				return false
 			}
-			goto nextTSID
+			continue
 		}
 
 		// Found the block with the given tsid. Verify timestamp range.
@@ -254,6 +285,7 @@ func (ps *partSearch) searchBHS() bool {
 		// So use linear search instead of binary search.
 		if bh.MaxTimestamp < ps.tr.MinTimestamp {
 			// Skip the block with too small timestamps.
+			bhs = bhs[1:]
 			continue
 		}
 		if bh.MinTimestamp > ps.tr.MaxTimestamp {
@@ -269,10 +301,9 @@ func (ps *partSearch) searchBHS() bool {
 		// Read it.
 		ps.BlockRef.init(ps.p, bh)
 
-		ps.bhs = ps.bhs[i+1:]
+		ps.bhs = bhs[1:]
 		return true
 	}
-
 	ps.bhs = nil
 	return false
 }
