@@ -2,61 +2,64 @@ package storage
 
 import (
 	"fmt"
-	"os"
 	"path/filepath"
 	"strings"
 	"sync"
 	"sync/atomic"
 	"time"
 
+	"github.com/VictoriaMetrics/VictoriaMetrics/lib/cgroup"
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/fasttime"
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/fs"
 	"github.com/VictoriaMetrics/VictoriaMetrics/lib/logger"
-	"github.com/VictoriaMetrics/VictoriaMetrics/lib/uint64set"
+	"github.com/VictoriaMetrics/VictoriaMetrics/lib/timeutil"
+	"github.com/valyala/fastrand"
 )
 
+var finalDedupScheduleInterval = time.Hour
+
+// SetFinalDedupScheduleInterval configures the interval for checking when the final deduplication process should start.
+func SetFinalDedupScheduleInterval(d time.Duration) {
+	finalDedupScheduleInterval = d
+}
+
 // table represents a single table with time series data.
 type table struct {
 	path                string
 	smallPartitionsPath string
 	bigPartitionsPath   string
 
-	getDeletedMetricIDs func() *uint64set.Set
-	retentionMsecs      int64
-	isReadOnly          *uint32
+	s *Storage
 
 	ptws     []*partitionWrapper
 	ptwsLock sync.Mutex
 
-	flockF *os.File
-
-	stop chan struct{}
+	stopCh chan struct{}
 
 	retentionWatcherWG  sync.WaitGroup
 	finalDedupWatcherWG sync.WaitGroup
+	forceMergeWG        sync.WaitGroup
 }
 
 // partitionWrapper provides refcounting mechanism for the partition.
 type partitionWrapper struct {
-	// Atomic counters must be at the top of struct for proper 8-byte alignment on 32-bit archs.
-	// See https://github.com/VictoriaMetrics/VictoriaMetrics/issues/212
+	// refCount is the number of open references to partitionWrapper.
+	refCount atomic.Int32
 
-	refCount uint64
-
-	// The partition must be dropped if mustDrop > 0
-	mustDrop uint64
+	// if mustDrop is true, then the partition must be dropped after refCount reaches zero.
+	mustDrop atomic.Bool
 
 	pt *partition
 }
 
 func (ptw *partitionWrapper) incRef() {
-	atomic.AddUint64(&ptw.refCount, 1)
+	ptw.refCount.Add(1)
 }
 
 func (ptw *partitionWrapper) decRef() {
-	n := atomic.AddUint64(&ptw.refCount, ^uint64(0))
-	if int64(n) < 0 {
-		logger.Panicf("BUG: pts.refCount must be positive; got %d", int64(n))
+	n := ptw.refCount.Add(-1)
+	if n < 0 {
+		logger.Panicf("BUG: pts.refCount must be positive; got %d", n)
 	}
 	if n > 0 {
 		return
@@ -65,106 +68,83 @@ func (ptw *partitionWrapper) decRef() {
 	// refCount is zero. Close the partition.
 	ptw.pt.MustClose()
 
-	if atomic.LoadUint64(&ptw.mustDrop) == 0 {
+	if !ptw.mustDrop.Load() {
 		ptw.pt = nil
 		return
 	}
 
-	// ptw.mustDrop > 0. Drop the partition.
+	// Drop the partition.
 	ptw.pt.Drop()
 	ptw.pt = nil
 }
 
 func (ptw *partitionWrapper) scheduleToDrop() {
-	atomic.AddUint64(&ptw.mustDrop, 1)
+	ptw.mustDrop.Store(true)
 }
 
-// openTable opens a table on the given path with the given retentionMsecs.
+// mustOpenTable opens a table on the given path.
 //
 // The table is created if it doesn't exist.
-//
-// Data older than the retentionMsecs may be dropped at any time.
-func openTable(path string, getDeletedMetricIDs func() *uint64set.Set, retentionMsecs int64, isReadOnly *uint32) (*table, error) {
+func mustOpenTable(path string, s *Storage) *table {
 	path = filepath.Clean(path)
 
 	// Create a directory for the table if it doesn't exist yet.
-	if err := fs.MkdirAllIfNotExist(path); err != nil {
-		return nil, fmt.Errorf("cannot create directory for table %q: %w", path, err)
-	}
-
-	// Protect from concurrent opens.
-	flockF, err := fs.CreateFlockFile(path)
-	if err != nil {
-		return nil, err
-	}
+	fs.MustMkdirIfNotExist(path)
 
 	// Create directories for small and big partitions if they don't exist yet.
-	smallPartitionsPath := path + "/small"
-	if err := fs.MkdirAllIfNotExist(smallPartitionsPath); err != nil {
-		return nil, fmt.Errorf("cannot create directory for small partitions %q: %w", smallPartitionsPath, err)
-	}
-	smallSnapshotsPath := smallPartitionsPath + "/snapshots"
-	if err := fs.MkdirAllIfNotExist(smallSnapshotsPath); err != nil {
-		return nil, fmt.Errorf("cannot create %q: %w", smallSnapshotsPath, err)
-	}
-	bigPartitionsPath := path + "/big"
-	if err := fs.MkdirAllIfNotExist(bigPartitionsPath); err != nil {
-		return nil, fmt.Errorf("cannot create directory for big partitions %q: %w", bigPartitionsPath, err)
-	}
-	bigSnapshotsPath := bigPartitionsPath + "/snapshots"
-	if err := fs.MkdirAllIfNotExist(bigSnapshotsPath); err != nil {
-		return nil, fmt.Errorf("cannot create %q: %w", bigSnapshotsPath, err)
-	}
+	smallPartitionsPath := filepath.Join(path, smallDirname)
+	fs.MustMkdirIfNotExist(smallPartitionsPath)
+	fs.MustRemoveTemporaryDirs(smallPartitionsPath)
+
+	smallSnapshotsPath := filepath.Join(smallPartitionsPath, snapshotsDirname)
+	fs.MustMkdirIfNotExist(smallSnapshotsPath)
+	fs.MustRemoveTemporaryDirs(smallSnapshotsPath)
+
+	bigPartitionsPath := filepath.Join(path, bigDirname)
+	fs.MustMkdirIfNotExist(bigPartitionsPath)
+	fs.MustRemoveTemporaryDirs(bigPartitionsPath)
+
+	bigSnapshotsPath := filepath.Join(bigPartitionsPath, snapshotsDirname)
+	fs.MustMkdirIfNotExist(bigSnapshotsPath)
+	fs.MustRemoveTemporaryDirs(bigSnapshotsPath)
 
 	// Open partitions.
-	pts, err := openPartitions(smallPartitionsPath, bigPartitionsPath, getDeletedMetricIDs, retentionMsecs, isReadOnly)
-	if err != nil {
-		return nil, fmt.Errorf("cannot open partitions in the table %q: %w", path, err)
-	}
+	pts := mustOpenPartitions(smallPartitionsPath, bigPartitionsPath, s)
 
 	tb := &table{
 		path:                path,
 		smallPartitionsPath: smallPartitionsPath,
 		bigPartitionsPath:   bigPartitionsPath,
-		getDeletedMetricIDs: getDeletedMetricIDs,
-		retentionMsecs:      retentionMsecs,
-		isReadOnly:          isReadOnly,
+		s:                   s,
 
-		flockF: flockF,
-
-		stop: make(chan struct{}),
+		stopCh: make(chan struct{}),
 	}
 	for _, pt := range pts {
 		tb.addPartitionNolock(pt)
 	}
 	tb.startRetentionWatcher()
 	tb.startFinalDedupWatcher()
-	return tb, nil
+	return tb
 }
 
-// CreateSnapshot creates tb snapshot and returns paths to small and big parts of it.
-func (tb *table) CreateSnapshot(snapshotName string) (string, string, error) {
+// MustCreateSnapshot creates tb snapshot and returns paths to small and big parts of it.
+func (tb *table) MustCreateSnapshot(snapshotName string) (string, string) {
 	logger.Infof("creating table snapshot of %q...", tb.path)
 	startTime := time.Now()
 
 	ptws := tb.GetPartitions(nil)
 	defer tb.PutPartitions(ptws)
 
-	dstSmallDir := fmt.Sprintf("%s/small/snapshots/%s", tb.path, snapshotName)
-	if err := fs.MkdirAllFailIfExist(dstSmallDir); err != nil {
-		return "", "", fmt.Errorf("cannot create dir %q: %w", dstSmallDir, err)
-	}
-	dstBigDir := fmt.Sprintf("%s/big/snapshots/%s", tb.path, snapshotName)
-	if err := fs.MkdirAllFailIfExist(dstBigDir); err != nil {
-		return "", "", fmt.Errorf("cannot create dir %q: %w", dstBigDir, err)
-	}
+	dstSmallDir := filepath.Join(tb.path, smallDirname, snapshotsDirname, snapshotName)
+	fs.MustMkdirFailIfExist(dstSmallDir)
+
+	dstBigDir := filepath.Join(tb.path, bigDirname, snapshotsDirname, snapshotName)
+	fs.MustMkdirFailIfExist(dstBigDir)
 
 	for _, ptw := range ptws {
-		smallPath := dstSmallDir + "/" + ptw.pt.name
-		bigPath := dstBigDir + "/" + ptw.pt.name
-		if err := ptw.pt.CreateSnapshotAt(smallPath, bigPath); err != nil {
-			return "", "", fmt.Errorf("cannot create snapshot for partition %q in %q: %w", ptw.pt.name, tb.path, err)
-		}
+		smallPath := filepath.Join(dstSmallDir, ptw.pt.name)
+		bigPath := filepath.Join(dstBigDir, ptw.pt.name)
+		ptw.pt.MustCreateSnapshotAt(smallPath, bigPath)
 	}
 
 	fs.MustSyncPath(dstSmallDir)
@@ -173,31 +153,32 @@ func (tb *table) CreateSnapshot(snapshotName string) (string, string, error) {
 	fs.MustSyncPath(filepath.Dir(dstBigDir))
 
 	logger.Infof("created table snapshot for %q at (%q, %q) in %.3f seconds", tb.path, dstSmallDir, dstBigDir, time.Since(startTime).Seconds())
-	return dstSmallDir, dstBigDir, nil
+	return dstSmallDir, dstBigDir
 }
 
 // MustDeleteSnapshot deletes snapshot with the given snapshotName.
 func (tb *table) MustDeleteSnapshot(snapshotName string) {
-	smallDir := fmt.Sprintf("%s/small/snapshots/%s", tb.path, snapshotName)
-	fs.MustRemoveAll(smallDir)
-	bigDir := fmt.Sprintf("%s/big/snapshots/%s", tb.path, snapshotName)
-	fs.MustRemoveAll(bigDir)
+	smallDir := filepath.Join(tb.path, smallDirname, snapshotsDirname, snapshotName)
+	fs.MustRemoveDirAtomic(smallDir)
+	bigDir := filepath.Join(tb.path, bigDirname, snapshotsDirname, snapshotName)
+	fs.MustRemoveDirAtomic(bigDir)
 }
 
 func (tb *table) addPartitionNolock(pt *partition) {
 	ptw := &partitionWrapper{
-		pt:       pt,
-		refCount: 1,
+		pt: pt,
 	}
+	ptw.incRef()
 	tb.ptws = append(tb.ptws, ptw)
 }
 
 // MustClose closes the table.
 // It is expected that all the pending searches on the table are finished before calling MustClose.
 func (tb *table) MustClose() {
-	close(tb.stop)
+	close(tb.stopCh)
 	tb.retentionWatcherWG.Wait()
 	tb.finalDedupWatcherWG.Wait()
+	tb.forceMergeWG.Wait()
 
 	tb.ptwsLock.Lock()
 	ptws := tb.ptws
@@ -205,45 +186,65 @@ func (tb *table) MustClose() {
 	tb.ptwsLock.Unlock()
 
 	for _, ptw := range ptws {
-		if n := atomic.LoadUint64(&ptw.refCount); n != 1 {
+		if n := ptw.refCount.Load(); n != 1 {
 			logger.Panicf("BUG: unexpected refCount=%d when closing the partition; probably there are pending searches", n)
 		}
 		ptw.decRef()
 	}
-
-	// Release exclusive lock on the table.
-	if err := tb.flockF.Close(); err != nil {
-		logger.Panicf("FATAL: cannot release lock on %q: %s", tb.flockF.Name(), err)
-	}
 }
 
-// flushRawRows flushes all the pending rows, so they become visible to search.
+// flushPendingRows flushes all the pending raw rows, so they become visible to search.
 //
 // This function is for debug purposes only.
-func (tb *table) flushRawRows() {
+func (tb *table) flushPendingRows() {
 	ptws := tb.GetPartitions(nil)
 	defer tb.PutPartitions(ptws)
 
 	for _, ptw := range ptws {
-		ptw.pt.flushRawRows(true)
+		ptw.pt.flushPendingRows(true)
 	}
 }
 
+func (tb *table) NotifyReadWriteMode() {
+	tb.ptwsLock.Lock()
+	for _, ptw := range tb.ptws {
+		ptw.pt.NotifyReadWriteMode()
+	}
+	tb.ptwsLock.Unlock()
+}
+
 // TableMetrics contains essential metrics for the table.
 type TableMetrics struct {
 	partitionMetrics
 
+	// LastPartition contains metrics for the last partition.
+	// These metrics are important, since the majority of data ingestion
+	// and querying goes to the last partition.
+	LastPartition partitionMetrics
+
 	PartitionsRefCount uint64
 }
 
 // UpdateMetrics updates m with metrics from tb.
 func (tb *table) UpdateMetrics(m *TableMetrics) {
-	tb.ptwsLock.Lock()
-	for _, ptw := range tb.ptws {
+	ptws := tb.GetPartitions(nil)
+	defer tb.PutPartitions(ptws)
+
+	for _, ptw := range ptws {
 		ptw.pt.UpdateMetrics(&m.partitionMetrics)
-		m.PartitionsRefCount += atomic.LoadUint64(&ptw.refCount)
+		m.PartitionsRefCount += uint64(ptw.refCount.Load())
+	}
+
+	// Collect separate metrics for the last partition.
+	if len(ptws) > 0 {
+		ptwLast := ptws[0]
+		for _, ptw := range ptws[1:] {
+			if ptw.pt.tr.MinTimestamp > ptwLast.pt.tr.MinTimestamp {
+				ptwLast = ptw
+			}
+		}
+		ptwLast.pt.UpdateMetrics(&m.LastPartition)
 	}
-	tb.ptwsLock.Unlock()
 }
 
 // ForceMergePartitions force-merges partitions in tb with names starting from the given partitionNamePrefix.
@@ -252,13 +253,17 @@ func (tb *table) UpdateMetrics(m *TableMetrics) {
 func (tb *table) ForceMergePartitions(partitionNamePrefix string) error {
 	ptws := tb.GetPartitions(nil)
 	defer tb.PutPartitions(ptws)
+
+	tb.forceMergeWG.Add(1)
+	defer tb.forceMergeWG.Done()
+
 	for _, ptw := range ptws {
 		if !strings.HasPrefix(ptw.pt.name, partitionNamePrefix) {
 			continue
 		}
 		logger.Infof("starting forced merge for partition %q", ptw.pt.name)
 		startTime := time.Now()
-		if err := ptw.pt.ForceMergeAllParts(); err != nil {
+		if err := ptw.pt.ForceMergeAllParts(tb.stopCh); err != nil {
 			return fmt.Errorf("cannot complete forced merge for partition %q: %w", ptw.pt.name, err)
 		}
 		logger.Infof("forced merge for partition %q has been finished in %.3f seconds", ptw.pt.name, time.Since(startTime).Seconds())
@@ -266,10 +271,10 @@ func (tb *table) ForceMergePartitions(partitionNamePrefix string) error {
 	return nil
 }
 
-// AddRows adds the given rows to the table tb.
-func (tb *table) AddRows(rows []rawRow) error {
+// MustAddRows adds the given rows to the table tb.
+func (tb *table) MustAddRows(rows []rawRow) {
 	if len(rows) == 0 {
-		return nil
+		return
 	}
 
 	// Verify whether all the rows may be added to a single partition.
@@ -306,7 +311,7 @@ func (tb *table) AddRows(rows []rawRow) error {
 		// Fast path - add all the rows into the ptw.
 		ptw.pt.AddRows(rows)
 		tb.PutPartitions(ptws)
-		return nil
+		return
 	}
 
 	// Slower path - split rows into per-partition buckets.
@@ -332,7 +337,7 @@ func (tb *table) AddRows(rows []rawRow) error {
 	}
 	tb.PutPartitions(ptws)
 	if len(missingRows) == 0 {
-		return nil
+		return
 	}
 
 	// The slowest path - there are rows that don't fit any existing partition.
@@ -361,23 +366,16 @@ func (tb *table) AddRows(rows []rawRow) error {
 			continue
 		}
 
-		pt, err := createPartition(r.Timestamp, tb.smallPartitionsPath, tb.bigPartitionsPath, tb.getDeletedMetricIDs, tb.retentionMsecs, tb.isReadOnly)
-		if err != nil {
-			// Return only the first error, since it has no sense in returning all errors.
-			tb.ptwsLock.Unlock()
-			return fmt.Errorf("errors while adding rows to table %q: %w", tb.path, err)
-		}
+		pt := mustCreatePartition(r.Timestamp, tb.smallPartitionsPath, tb.bigPartitionsPath, tb.s)
 		pt.AddRows(missingRows[i : i+1])
 		tb.addPartitionNolock(pt)
 	}
 	tb.ptwsLock.Unlock()
-
-	return nil
 }
 
 func (tb *table) getMinMaxTimestamps() (int64, int64) {
 	now := int64(fasttime.UnixTimestamp() * 1000)
-	minTimestamp := now - tb.retentionMsecs
+	minTimestamp := now - tb.s.retentionMsecs
 	maxTimestamp := now + 2*24*3600*1000 // allow max +2 days from now due to timezones shit :)
 	if minTimestamp < 0 {
 		// Negative timestamps aren't supported by the storage.
@@ -398,16 +396,17 @@ func (tb *table) startRetentionWatcher() {
 }
 
 func (tb *table) retentionWatcher() {
-	ticker := time.NewTicker(time.Minute)
+	d := timeutil.AddJitterToDuration(time.Minute)
+	ticker := time.NewTicker(d)
 	defer ticker.Stop()
 	for {
 		select {
-		case <-tb.stop:
+		case <-tb.stopCh:
 			return
 		case <-ticker.C:
 		}
 
-		minTimestamp := int64(fasttime.UnixTimestamp()*1000) - tb.retentionMsecs
+		minTimestamp := int64(fasttime.UnixTimestamp()*1000) - tb.s.retentionMsecs
 		var ptwsDrop []*partitionWrapper
 		tb.ptwsLock.Lock()
 		dst := tb.ptws[:0]
@@ -425,7 +424,7 @@ func (tb *table) retentionWatcher() {
 			continue
 		}
 
-		// There are paritions to drop. Drop them.
+		// There are partitions to drop. Drop them.
 
 		// Remove table references from partitions, so they will be eventually
 		// closed and dropped after all the pending searches are done.
@@ -454,22 +453,41 @@ func (tb *table) finalDedupWatcher() {
 		defer tb.PutPartitions(ptws)
 		timestamp := timestampFromTime(time.Now())
 		currentPartitionName := timestampToPartitionName(timestamp)
+		var ptwsToDedup []*partitionWrapper
 		for _, ptw := range ptws {
-			if ptw.pt.name == currentPartitionName || !ptw.pt.isFinalDedupNeeded() {
+			if ptw.pt.name == currentPartitionName {
 				// Do not run final dedup for the current month.
 				continue
 			}
-			if err := ptw.pt.runFinalDedup(); err != nil {
-				logger.Errorf("cannot run final dedup for partition %s: %s", ptw.pt.name, err)
+			if !ptw.pt.isFinalDedupNeeded() {
+				// There is no need to run final dedup for the given partition.
 				continue
 			}
+			// mark partition with final deduplication marker
+			ptw.pt.isDedupScheduled.Store(true)
+			ptwsToDedup = append(ptwsToDedup, ptw)
+		}
+		for _, ptw := range ptwsToDedup {
+			if err := ptw.pt.runFinalDedup(tb.stopCh); err != nil {
+				logger.Errorf("cannot run final dedup for partition %s: %s", ptw.pt.name, err)
+			}
+			ptw.pt.isDedupScheduled.Store(false)
 		}
 	}
-	t := time.NewTicker(time.Hour)
+
+	// adds 25% jitter in order to prevent thundering herd problem
+	// https://github.com/VictoriaMetrics/VictoriaMetrics/issues/7880
+	addJitter := func(d time.Duration) time.Duration {
+		dv := d / 4
+		p := float64(fastrand.Uint32()) / (1 << 32)
+		return d + time.Duration(p*float64(dv))
+	}
+	d := addJitter(finalDedupScheduleInterval)
+	t := time.NewTicker(d)
 	defer t.Stop()
 	for {
 		select {
-		case <-tb.stop:
+		case <-tb.stopCh:
 			return
 		case <-t.C:
 			f()
@@ -499,60 +517,55 @@ func (tb *table) PutPartitions(ptws []*partitionWrapper) {
 	}
 }
 
-func openPartitions(smallPartitionsPath, bigPartitionsPath string, getDeletedMetricIDs func() *uint64set.Set, retentionMsecs int64, isReadOnly *uint32) ([]*partition, error) {
+func mustOpenPartitions(smallPartitionsPath, bigPartitionsPath string, s *Storage) []*partition {
 	// Certain partition directories in either `big` or `small` dir may be missing
 	// after restoring from backup. So populate partition names from both dirs.
 	ptNames := make(map[string]bool)
-	if err := populatePartitionNames(smallPartitionsPath, ptNames); err != nil {
-		return nil, err
-	}
-	if err := populatePartitionNames(bigPartitionsPath, ptNames); err != nil {
-		return nil, err
-	}
+	mustPopulatePartitionNames(smallPartitionsPath, ptNames)
+	mustPopulatePartitionNames(bigPartitionsPath, ptNames)
 	var pts []*partition
+	var ptsLock sync.Mutex
+
+	// Open partitions in parallel. This should reduce the time needed for opening multiple partitions.
+	var wg sync.WaitGroup
+	concurrencyLimiterCh := make(chan struct{}, cgroup.AvailableCPUs())
 	for ptName := range ptNames {
-		smallPartsPath := smallPartitionsPath + "/" + ptName
-		bigPartsPath := bigPartitionsPath + "/" + ptName
-		pt, err := openPartition(smallPartsPath, bigPartsPath, getDeletedMetricIDs, retentionMsecs, isReadOnly)
-		if err != nil {
-			mustClosePartitions(pts)
-			return nil, fmt.Errorf("cannot open partition %q: %w", ptName, err)
-		}
-		pts = append(pts, pt)
+		wg.Add(1)
+		concurrencyLimiterCh <- struct{}{}
+		go func(ptName string) {
+			defer func() {
+				<-concurrencyLimiterCh
+				wg.Done()
+			}()
+
+			smallPartsPath := filepath.Join(smallPartitionsPath, ptName)
+			bigPartsPath := filepath.Join(bigPartitionsPath, ptName)
+			pt := mustOpenPartition(smallPartsPath, bigPartsPath, s)
+
+			ptsLock.Lock()
+			pts = append(pts, pt)
+			ptsLock.Unlock()
+		}(ptName)
 	}
-	return pts, nil
+	wg.Wait()
+
+	return pts
 }
 
-func populatePartitionNames(partitionsPath string, ptNames map[string]bool) error {
-	d, err := os.Open(partitionsPath)
-	if err != nil {
-		return fmt.Errorf("cannot open directory with partitions %q: %w", partitionsPath, err)
-	}
-	defer fs.MustClose(d)
-
-	fis, err := d.Readdir(-1)
-	if err != nil {
-		return fmt.Errorf("cannot read directory with partitions %q: %w", partitionsPath, err)
-	}
-	for _, fi := range fis {
-		if !fs.IsDirOrSymlink(fi) {
+func mustPopulatePartitionNames(partitionsPath string, ptNames map[string]bool) {
+	des := fs.MustReadDir(partitionsPath)
+	for _, de := range des {
+		if !fs.IsDirOrSymlink(de) {
 			// Skip non-directories
 			continue
 		}
-		ptName := fi.Name()
-		if ptName == "snapshots" {
+		ptName := de.Name()
+		if ptName == snapshotsDirname {
 			// Skip directory with snapshots
 			continue
 		}
 		ptNames[ptName] = true
 	}
-	return nil
-}
-
-func mustClosePartitions(pts []*partition) {
-	for _, pt := range pts {
-		pt.MustClose()
-	}
 }
 
 type partitionWrappers struct {
