@@ -5,9 +5,10 @@ import (
 	"time"
 )
 
+// dateToString returns human readable representation of the date.
 func dateToString(date uint64) string {
 	if date == 0 {
-		return "1970-01-01"
+		return "[entire retention period]"
 	}
 	t := time.Unix(int64(date*24*3600), 0).UTC()
 	return t.Format("2006-01-02")
@@ -33,7 +34,34 @@ type TimeRange struct {
 	MaxTimestamp int64
 }
 
+// Zero time range and zero date are used to force global index search.
+var (
+	globalIndexTimeRange = TimeRange{}
+	globalIndexDate      = uint64(0)
+)
+
+// DateRange returns the date range for the given time range.
+func (tr *TimeRange) DateRange() (uint64, uint64) {
+	minDate := uint64(tr.MinTimestamp) / msecPerDay
+	// Max timestamp may point to the first millisecond of the next day. As the
+	// result, the returned date range will cover one more day than needed.
+	// Decrementing by 1 removes this extra day.
+	maxDate := uint64(tr.MaxTimestamp-1) / msecPerDay
+
+	// However, if both timestamps are the same and point to the beginning of
+	// the day, then maxDate will be smaller that the minDate. In this case
+	// maxDate is set to minDate.
+	if maxDate < minDate {
+		maxDate = minDate
+	}
+
+	return minDate, maxDate
+}
+
 func (tr *TimeRange) String() string {
+	if *tr == globalIndexTimeRange {
+		return "[entire retention period]"
+	}
 	start := TimestampToHumanReadableFormat(tr.MinTimestamp)
 	end := TimestampToHumanReadableFormat(tr.MaxTimestamp)
 	return fmt.Sprintf("[%s..%s]", start, end)
